local MIN_SCALE = .6
local MAX_SCALE = 1.1

local TIMECHECK_CREATE = 0
local TIMECHECK_SAVE = 0

local skillLineXPBarFragment = ZO_FadeSceneFragment:New(ZO_GamepadAlchemyTopLevelSkillInfo)

local function getTimeElapsed(lastTime)
 	local elapsedTime = GetGameTimeMilliseconds() - lastTime
	return elapsedTime
end

local function areAlchemyPanelsEnabled()
 	return IJA_ACTIVEWRITS[CRAFTING_TYPE_ALCHEMY]
end
	
IJA_AlchemyHorizontalScrollList_Gamepad = ZO_HorizontalScrollList_Gamepad:Subclass()
function IJA_AlchemyHorizontalScrollList_Gamepad:New(...)
     return ZO_HorizontalScrollList_Gamepad.New(self, ...)
end

-------------------------------------
-- IJA_EasyAlchemy
-------------------------------------
local IJA_GamepadSavedAlchemyList = ZO_Object:MultiSubclass(ZO_GamepadCraftingInventory, IJA_EASYALCHEMY)
local IJA_GamepadEasyAlchemy = IJA_EASYALCHEMY:Subclass()

function IJA_GamepadEasyAlchemy:New(owner)
	local newObject = ZO_Object:MultiSubclass(self, owner)
	newObject:Initialize(owner)
	return newObject
end

function IJA_GamepadEasyAlchemy:Initialize(owner)
	self.selectedData = {
		['type'] = 1,
		['solvent'] = {},
		['traitIds'] = {[1] = 1}
	}
	
	self.traitLists = {}
	local control = IJA_GamepadEasyAlchemyTopLevel
	
	local panelControl = control.createControl
	local controlCreate =  panelControl:GetNamedChild("Container") -- "create" list control
	self.scrollContainer = panelControl:GetNamedChild("ScrollContainer")
    self.scrollChild = self.scrollContainer:GetNamedChild("ScrollChild")
	
	self.control = control
    self.controlCreate = controlCreate
	self.panelControl = panelControl
	
    self.refreshGroup = ZO_OrderedRefreshGroup:New(ZO_ORDERED_REFRESH_GROUP_AUTO_CLEAN_PER_FRAME)
    self.refreshGroup:AddDirtyState("AllLists", function() self:RefreshAllListsLayer() end)
    self.refreshGroup:AddDirtyState("Result", function() self:RefreshResultLayer()  end)
    self.refreshGroup:SetActive(function()
        return SCENE_MANAGER:IsShowing("gamepad_alchemy_creation") or SCENE_MANAGER:IsShowing("ija_gamepad_alchemy_saves")
    end)
	
	self:InitializeFragments(control)
	self:InitializeCraftingScene()
	
	self:InitializeHooks()
	self:InitializeModeBar()
	
    local function StateChange(oldState, newState)
        if newState == SCENE_SHOWING then
		elseif newState == SCENE_SHOWN then
			GAMEPAD_ALCHEMY_ROOT_SCENE:UnregisterCallback("StateChange", StateChange)
            self:PerformDeferredInitialization()
        elseif newState == SCENE_HIDDEN then
        end
    end
	
    GAMEPAD_ALCHEMY_ROOT_SCENE:RegisterCallback("StateChange", StateChange)
end

function IJA_GamepadEasyAlchemy:PerformDeferredInitialization()
     if self.mainKeybindStripDescriptor then return end
	self:SetMemory("solventLevel", self:GetSkillRank(CRAFTING_TYPE_ALCHEMY, 1))
	
    local scrollListControl = IJA_AlchemyHorizontalScrollList_Gamepad
    local listSlotTemplate = "ZO_GamepadSmithingListSlot"
	self:InitializeLists(scrollListControl, listSlotTemplate)
	
    self.movementController = ZO_MovementController:New(MOVEMENT_CONTROLLER_DIRECTION_VERTICAL)
	
	self.saves = IJA_GamepadSavedAlchemyList:New(self, self.control.savesControl)
	
    self:InitializeKeybindStripDescriptors()
	
    self:SetupListActivationFunctions()
    self:InitializeScrollPanel()
    self:InitializeFocusItems()
	
	self.control:SetHidden(true)
	
--	self.control:SetHandler("OnUpdate", function(_, currentFrameTimeSeconds) d(_,currentFrameTimeSeconds) end)
--	ZO_GamepadAlchemyTopLevelTooltipContainerTipScroll:SetHandler("OnUpdate", function(_, currentFrameTimeSeconds) d( ZO_GamepadAlchemyTopLevelTooltipContainerTipScroll:GetAnchor()) end)

-- /script ZO_GamepadAlchemyTopLevelTooltipContainerTipScroll:SetHandler("OnUpdate", function() d() end)
-- /script d( GAMEPAD_ALCHEMY.tooltip.tip)
end

function IJA_GamepadEasyAlchemy:InitializeFragments(control)
 	IJA_GAMEPAD_EASYALCHEMY_FRAGMENT = ZO_SimpleSceneFragment:New(control)
	IJA_GAMEPAD_EASYALCHEMY_FRAGMENT:SetHideOnSceneHidden(true)
	
	IJA_GAMEPAD_ALCHEMY_CRAFTLIST_FRAGMENT = ZO_CreateQuadrantConveyorFragment(control.createControl)
	IJA_GAMEPAD_ALCHEMY_SAVELIST_FRAGMENT = ZO_CreateQuadrantConveyorFragment(control.savesControl)
	IJA_GAMEPAD_ALCHEMY_SOLVENT_FRAGMENT = ZO_CreateQuadrantConveyorFragment(IJA_GamepadSavedAlchemySolventTopLevel:GetNamedChild("ContainerSolventList"))

	IJA_GAMEPAD_EASYALCHEMY_FRAGMENTGROUP = {
		skillLineXPBarFragment,
		IJA_ALCHEMY_FRAGMENT,
		IJA_GAMEPAD_EASYALCHEMY_FRAGMENT,
		GAMEPAD_CRAFTING_RESULTS_FRAGMENT,
		GAMEPAD_ALCHEMY_FRAGMENT,
		GAMEPAD_NAV_QUADRANT_1_BACKGROUND_FRAGMENT,
		MINIMIZE_CHAT_FRAGMENT,
		GAMEPAD_MENU_SOUND_FRAGMENT,
		CRAFTING_WINDOW_KEYBIND_INTERCEPT_LAYER_FRAGMENT
	}
end

function IJA_GamepadEasyAlchemy:InitializeCraftingScene()
	local TIME_LIMIT = 	1 * 60000 -- minutes to milliseconds
			 -- minutes ^
	local function resetStation(TIMECHECK)
 		if getTimeElapsed(TIMECHECK) > TIME_LIMIT + GetGameTimeMilliseconds() then
			self.alchemySelectionMemory = {}
		end
		TIMECHECK = GetGameTimeMilliseconds()
	end

	GAMEPAD_ALCHEMY_CREATION_SCENE:UnregisterAllCallbacks("StateChange")
	
	GAMEPAD_ALCHEMY_CREATION_SCENE = GAMEPAD_ALCHEMY:CreateInteractScene("ija_gamepad_alchemy_creation")
    GAMEPAD_ALCHEMY_CREATION_SCENE:SetInputPreferredMode(INPUT_PREFERRED_MODE_ALWAYS_GAMEPAD)
	
	GAMEPAD_ALCHEMY_CREATION_SCENE:AddFragment(IJA_RESULTPANEL_FRAGMENT)
	GAMEPAD_ALCHEMY_CREATION_SCENE:AddFragment(IJA_GAMEPAD_ALCHEMY_CRAFTLIST_FRAGMENT)
	
	GAMEPAD_ALCHEMY_CREATION_SCENE:AddFragmentGroup(IJA_GAMEPAD_EASYALCHEMY_FRAGMENTGROUP)
	
	GAMEPAD_ALCHEMY_CREATION_SCENE:AddFragmentGroup(FRAGMENT_GROUP.GAMEPAD_DRIVEN_UI_WINDOW)
	GAMEPAD_ALCHEMY_CREATION_SCENE:AddFragment(GAMEPAD_ALCHEMY_SLOTS_FRAGMENT)
	GAMEPAD_ALCHEMY_CREATION_SCENE:AddFragment(GAMEPAD_CRAFT_ADVISOR_FRAGMENT)

	GAMEPAD_ALCHEMY_CREATION_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		if newState == SCENE_SHOWING then
			ZO_GamepadAlchemyTopLevelHeaderContainerHeaderTitleContainerTitle:SetText(GetString(SI_IJAEA_EASYALCHEMY))
			IJA_RESULTPANEL_FRAGMENT:Hide()
		elseif newState == SCENE_SHOWN then
			self.focus:SetFocusByIndex(1) -- sets focus to top list
			self.focus:Activate()
			
			self:SetStationFromMemory("ModeMemory_1")

--			IJA_GamepadEasyAlchemyTopLevelContainerCreation:SetHidden(false)
		
--			resetStation(TIMECHECK_CREATE)
		
			local isWrit = false
			if IJA_ACTIVEWRITS and IJA_ACTIVEWRITS[CRAFTING_TYPE_ALCHEMY] then
				isWrit = true
			end
		
			if not isWrit then
--				self:RefreshSolventList(self.selectedData)
--				self:RefreshTraitLists(1)
			else
			end
			
			self.selectedData.type = self:GetMemory("selectedType") or 1
			self:SetResultPanelHeader(GetString(SI_IJAEA_EASYALCHEMY))
			
			KEYBIND_STRIP:RemoveDefaultExit()
			self:AddKeybindButtonGroup(self.mainKeybindStripDescriptor)
			
			self:RefreshResultType()
			
			self:DirtyAllLists()
			self.refreshGroup:TryClean()
		elseif newState == SCENE_HIDING then
	--	d( 'SCENE_HIDING')
		elseif newState == SCENE_HIDDEN then
			self:SetStationMemory("ModeMemory_1")
			self.focus:Deactivate()
			
			KEYBIND_STRIP:RemoveKeybindButtonGroup(self.mainKeybindStripDescriptor)
			KEYBIND_STRIP:RestoreDefaultExit()
			
			GAMEPAD_CRAFTING_RESULTS:SetCraftingTooltip(nil)
			GAMEPAD_ALCHEMY.tooltip:SetHidden(true)
		end
	end)

	self:AddAlchemyCraftingScene("ija_gamepad_alchemy_creation")
end

function IJA_GamepadEasyAlchemy:InitializeHooks()
	local function isNotActiveWrit()
		if IJA_ACTIVEWRITS then 
			return not IJA_ACTIVEWRITS[CRAFTING_TYPE_ALCHEMY]
		end
		-- refresh
		return true
	end

 	local function activateLists(activate)
 		self.typeList:SetEnabled(activate)
		for i=1, 3 do self.traitLists[i]:SetEnabled(activate) end
		self.solventList:SetEnabled(activate)
	end
	
	local function showElements(mode)
 		IJA_GamepadEasyAlchemyTopLevel:SetHidden(mode == ZO_ALCHEMY_MODE_RECIPES)
		IJA_GamepadEasyAlchemyTopLevelContainerCreation:SetHidden(mode ~= ZO_ALCHEMY_MODE_CREATION)
		IJA_GamepadEasyAlchemyTopLevelContainerSaves:SetHidden(mode ~= IJ_ALCHEMY_MODE_SAVES)
	end

	local isLocal = false
	local SelectMode = {
		['mode'] = 1,
		['index'] = 2,
		['type'] = "SelectMode",
		['doFunc'] = function(mode)
	--		showElements(mode)
			if mode ~= ZO_ALCHEMY_MODE_RECIPES then
				isLocal = true
				ApplyTemplateToControl(ZO_GamepadAlchemyTopLevelSlotContainer, "IJA_GamepadAlchemyTopLevelSlotContainer")
				if mode == ZO_ALCHEMY_MODE_CREATION then
					SCENE_MANAGER:Push("ija_gamepad_alchemy_creation")
		--			activateLists(isNotActiveWrit())
				else -- mode is IJ_ALCHEMY_MODE_SAVES
					SCENE_MANAGER:Push("ija_gamepad_alchemy_saves")
				end
				return true
			else
				ApplyTemplateToControl(ZO_GamepadAlchemyTopLevelSlotContainer, "ZO_GamepadCraftingIngredientBarTemplate")
			end
			isLocal = false
			return false
		end
	}
	self:AddCallBack(SelectMode)
	
	local StateChange = {
		['mode'] = 1,
		['index'] = 2,
		['type'] = "StateChange",
		['doFunc'] = function(oldState, newState)
			if newState == SCENE_SHOWING then
				self:UpdateModeBar()
				self.tooltip = GAMEPAD_ALCHEMY.tooltip
				GAMEPAD_CRAFTING_RESULTS:SetCraftingTooltip(self.tooltip)
				GAMEPAD_CRAFTING_RESULTS:SetTooltipAnimationSounds(SOUNDS.ALCHEMY_CREATE_TOOLTIP_GLOW_SUCCESS, SOUNDS.ALCHEMY_CREATE_TOOLTIP_GLOW_FAIL)
			elseif newState == SCENE_HIDDEN then
				GAMEPAD_CRAFTING_RESULTS:SetCraftingTooltip(nil)
				self.tooltip:SetHidden(true)
				
			end
		--	return true
		end
	}
	self:AddCallBack(StateChange)
end

function IJA_GamepadEasyAlchemy:InitializeModeBar()
 	local data = ZO_GamepadEntryData:New(GetString(SI_IJAEA_SAVEDRECIPES), "/esoui/art/inventory/gamepad/gp_inventory_icon_consumables.dds")
	data.mode = IJ_ALCHEMY_MODE_SAVES
	GAMEPAD_ALCHEMY.modeList:AddEntry("ZO_GamepadItemEntryTemplate", data)
	
	local newOrder = {}
	newOrder[1] = GAMEPAD_ALCHEMY.modeList.dataList[1]
	newOrder[2] = GAMEPAD_ALCHEMY.modeList.dataList[3]
	newOrder[3] = GAMEPAD_ALCHEMY.modeList.dataList[2]
	GAMEPAD_ALCHEMY.modeList.dataList = newOrder
	
	GAMEPAD_ALCHEMY.modeList:Commit()
end

function IJA_GamepadEasyAlchemy:UpdateModeBar()
	GAMEPAD_ALCHEMY.modeList.dataList[1]:ClearIcons()
	
	local alchemyQuestInfo = self:GetQuestInfo()
	if alchemyQuestInfo.materialItemId and alchemyQuestInfo.hasDesiredPotion == nil then
		GAMEPAD_ALCHEMY.modeList.dataList[1]:AddIcon("EsoUI/Art/WritAdvisor/Gamepad/gp_advisor_trackedPin_icon.dds")
		GAMEPAD_ALCHEMY.modeList.dataList[1]:AddIcon("EsoUI/Art/Crafting/Gamepad/gp_crafting_menuIcon_create.dds")
	else
		GAMEPAD_ALCHEMY.modeList.dataList[1]:AddIcon("EsoUI/Art/Crafting/Gamepad/gp_crafting_menuIcon_create.dds")
	end
	
	GAMEPAD_ALCHEMY.modeList:Commit()
end

function IJA_GamepadEasyAlchemy:InitializeKeybindStripDescriptors()
 	-- Main keybind strip
    self.mainKeybindStripDescriptor =
    {
        alignment = KEYBIND_STRIP_ALIGN_LEFT,
       
        {-- Perform craft
            name = function()
                local cost = GetCostToCraftAlchemyItem(GAMEPAD_ALCHEMY.solventSlot:GetBagAndSlot())
                return ZO_CraftingUtils_GetCostToCraftString(cost)
            end,
            keybind = "UI_SHORTCUT_SECONDARY",
            gamepadOrder = 1000,
            callback = function()
				if SCENE_MANAGER:IsShowing('ija_gamepad_alchemy_saves') and self.selectedSave then
					self.saves.list:SetSelectedIndex(self.selectedSave, ALLOW_EVEN_IF_DISABLED, SKIP_ANIMATION)
				end
                GAMEPAD_ALCHEMY:Create(1)
            end,
            enabled = function()
                return GAMEPAD_ALCHEMY:ShouldCraftButtonBeEnabled() and not self:IsUpdateInProgress()
            end,
            sound = SOUNDS.GAMEPAD_ALCHEMY_BEGIN,
        },
        
        {-- Multicraft
            name = GetString(SI_GAMEPAD_CRAFT_MULTIPLE),
            keybind = "UI_SHORTCUT_QUATERNARY",
            gamepadOrder = 1010,
            callback = function()
                local itemLink, _ = GetAlchemyResultingItemLink(GAMEPAD_ALCHEMY:GetAllCraftingBagAndSlots())
                ZO_GamepadCraftingUtils_ShowMultiCraftDialog(GAMEPAD_ALCHEMY, itemLink)
            end,
            enabled = function()
                return GAMEPAD_ALCHEMY:ShouldMultiCraftButtonBeEnabled() and GAMEPAD_ALCHEMY:ShouldCraftButtonBeEnabled() and not self:IsUpdateInProgress()
            end,
        },
		
        {-- Save or Delete Recipe
			name = GetString(SI_SAVE),
			keybind = "UI_SHORTCUT_TERTIARY",
            gamepadOrder = 1020,
			enabled = function()
				return self:IsSaveButtonEnabled()
			end,
			callback = function()
				self:SaveCurrentRecipe()
			end,
			visible = function()
				return self:IsShowingAlchemyCraftingScene()
			end,
        },
    }
    ZO_GamepadCraftingUtils_AddGenericCraftingBackKeybindsToDescriptor(self.mainKeybindStripDescriptor)
    ZO_CraftingUtils_ConnectKeybindButtonGroupToCraftingProcess(self.mainKeybindStripDescriptor)
end

function IJA_GamepadEasyAlchemy:RefreshAllListsLayer()
	self:UpdateModeBar()
	if not self.typeList then return end
--	self.typeList:RefreshVisible()
	self:RefreshResultType()
	self.solventList:RefreshVisible()
	for i=1, 3 do self.traitLists[i]:RefreshVisible() end
end

local function OnHorizonalScrollListShown(list)
     local listContainer = list:GetControl():GetParent()
    listContainer.selectedLabel:SetHidden(false)
end

function IJA_GamepadEasyAlchemy:RefreshResultLayer()
     self:UpdateTooltip(self:GetTooltipData())
    KEYBIND_STRIP:UpdateKeybindButtonGroup(self.mainKeybindStripDescriptor)
end

function IJA_GamepadEasyAlchemy:InitializeList(listName, TYPE, scrollListClass, listSlotTemplate)
     local listContainer = self.controlCreate:GetNamedChild(listName)
	
    listContainer.titleLabel:SetText(GetString("SI_IJAEA_EASYALCHEMY_HEADER", TYPE))
	listContainer.titleLabel:SetFont("ZoFontGamepad42")
	
    local function SetupFunction(control, data, selected, selectedDuringRebuild, enabled)
		control:GetNamedChild('QuestPin'):SetHidden(not data.questPinState)
 	
		----------------------------------------------
        ZO_ItemSlot_SetupSlot(control, 1, data.icon, true, not enabled)
			
        if selected then
            listContainer.selectedLabel:SetText(zo_strformat(SI_SMITHING_SELECTED_PATTERN, data.name))
			self:SetLabelHidden(listContainer.extraInfoLabel, true)
			
			if listName == "TypeList" then
	--			listContainer.titleLabel:SetText(GetString("SI_IJAEA_EASYALCHEMY_HEADER_SOLVENT", data.solventType))
			end
			if listName == "SolventList" then
			--	listContainer.titleLabel:SetText(GetString("SI_IJAEA_EASYALCHEMY_HEADER_SOLVENT", data.solventType))
 --                   listContainer.extraInfoLabel:SetColor(ZO_ERROR_COLOR:UnpackRGBA())
             --       listContainer.extraInfoLabel:SetText("# " .. data.stackCount)
 --                   listContainer.extraInfoLabel:SetColor(ZO_SUCCEEDED_TEXT:UnpackRGBA())
			end
        end
    end

    local function EqualityFunction(leftData, rightData)
         return leftData.traitId == rightData.traitId
    end

    local function OnHorizonalScrollListCleared(...)
         self:OnHorizonalScrollListCleared(...)
    end

    scrollList = scrollListClass:New(listContainer.listControl, listSlotTemplate, 5, SetupFunction, EqualityFunction, OnHorizonalScrollListShown, OnHorizonalScrollListCleared)

    scrollList:SetScaleExtents(MIN_SCALE, MAX_SCALE)
	scrollList.displayEntryType = ZO_HORIZONTAL_SCROLL_LIST_ANCHOR_ENTRIES_AT_FIXED_DISTANCE
	scrollList.offsetBetweenEntries = 50
	
--	scrollList:SetMouseEnabled(true)

    ZO_CraftingUtils_ConnectHorizontalScrollListToCraftingProcess(scrollList)
	return scrollList
end

function IJA_GamepadEasyAlchemy:InitializeLists(scrollListClass, listSlotTemplate)
 	local typeList = self:InitializeList("TypeList", IJA_ALCHEMY_FILTER_TYPE, scrollListClass, listSlotTemplate)
	typeList:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedTypeDataChangedCallback(selectedData, oldData, selectedDuringRebuild)
    end)
	scrollList.offsetBetweenEntries = 400
	self.typeList = typeList
	
	local solventList = self:InitializeList("SolventList", IJA_ALCHEMY_FILTER_SOLVENT, scrollListClass, listSlotTemplate)
	solventList:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedSolventDataChangedCallback(selectedData, oldData, selectedDuringRebuild)
    end)
	self.solventList = solventList
	local trait_1List = self:InitializeList("Trait_1List", IJA_ALCHEMY_FILTER_TRAIT, scrollListClass, listSlotTemplate)
	trait_1List:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedTraitDataChangedCallback(trait_1List, 1, selectedData, oldData, selectedDuringRebuild)
    end)
	self.traitLists[1] = trait_1List

	local trait_2List = self:InitializeList("Trait_2List", IJA_ALCHEMY_FILTER_TRAIT + 1, scrollListClass, listSlotTemplate)
	trait_2List:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedTraitDataChangedCallback(trait_2List, 2, selectedData, oldData, selectedDuringRebuild)
    end)
	self.traitLists[2] = trait_2List
	
	local trait_3List = self:InitializeList("Trait_3List", IJA_ALCHEMY_FILTER_TRAIT + 2, scrollListClass, listSlotTemplate)
	trait_3List:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedTraitDataChangedCallback(trait_3List, 3, selectedData, oldData, selectedDuringRebuild)
		self:UpdateTooltip(self:GetTooltipData())
    end)
	self.traitLists[3] = trait_3List
	
end

function IJA_GamepadEasyAlchemy:OnSelectedTypeDataChangedCallback(selectedData, oldData, selectedDuringRebuild)
 	self.selectedData.type = selectedData.solventType
	
	if not selectedDuringRebuild then
		self:SetMemory("selectedType", self.selectedData.type)
	end
					
	self:RefreshSolventList(self.selectedData)
	self:RefreshTraitLists(1)
end

function IJA_GamepadEasyAlchemy:RefreshResultType()
	local alchemyQuestInfo = self.alchemyQuestInfo
	local questSolventType = 0
	
	if alchemyQuestInfo and not alchemyQuestInfo.hasDesiredPotion then
		questSolventType = self:GetBaseIsPoison(alchemyQuestInfo.basePotionItemId) and 2 or 1
	end
	
 	self.typeList:Clear()
	for solventType = 1, 2 do
		local name = solventType == 1 and GetString(SI_ITEMTYPE7) or GetString(SI_ITEMTYPE30)
		local icon = solventType == 1 and "/esoui/art/icons/crownpotion_trires.dds" or "/esoui/art/icons/crownpoison_lethal.dds"
		
		questPinState = questSolventTypequestSolventType == solventType
		self.typeList:AddEntry({solventType = solventType, name = name, icon = icon, filterType = IJA_ALCHEMY_FILTER_TYPE, questPinState = questPinState})
	end
	
	self.typeList:Commit()
	
	if questSolventType ~= 0 then
		local initialListIndex = 0
		local index = self.typeList:FindIndexFromData(questSolventType, function(oldSolventType, newData)
			return oldSolventType == newData.solventType
		end)
		if index then
			initialListIndex = index
		end
	
		local ALLOW_EVEN_IF_DISABLED = true
		local SKIP_ANIMATION = false
	
		self.typeList:SetSelectedIndex(initialListIndex, ALLOW_EVEN_IF_DISABLED, SKIP_ANIMATION)
	end
end

function IJA_GamepadEasyAlchemy:DirtyAllLists()
     self.refreshGroup:MarkDirty("AllLists")
end

function IJA_GamepadEasyAlchemy:DirtyResult()
     self.refreshGroup:MarkDirty("Result")
end

-------------------------------------
-- 
-------------------------------------
do
	local selectedColor = ZO_ColorDef:New(GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_SELECTED))
	local disabledColor = ZO_ColorDef:New(GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_DISABLED))
	local COLOR_TABLE = {
		[true] = selectedColor,
		[false] = disabledColor,
	}
	local function ActivationChangedFn(list, activated)
 		local parentControl = list:GetControl():GetParent()
		parentControl.selectedLabel:SetColor(COLOR_TABLE[activated]:UnpackRGBA())
	end

    function IJA_GamepadEasyAlchemy:SetupListActivationFunctions()
         local lists = {self.typeList, self.solventList, self.traitLists[1], self.traitLists[2], self.traitLists[3]}
        for _, entry in pairs(lists) do
            entry:SetOnActivatedChangedFunction(ActivationChangedFn)
            ActivationChangedFn(entry, false)
        end
    end
end

function IJA_GamepadEasyAlchemy:RefreshAvailableFilters(dontReselect)	--------------------------------------
--	self:DirtyAllLists()
end

do
    local ACTIVE = true
    function IJA_GamepadEasyAlchemy:InitializeFocusItems()
 	
        self.focus = ZO_GamepadFocus:New(self.controlCreate)
		
        self.activateFocusFunction = function(focus, data)
             self.selectedList = focus
            focus:Activate()
            self:UpdateScrollPanel(focus)
--			self.resultPanel:UpdateKeybindStrip()
            self:UpdateBorderHighlight(focus, ACTIVE)
        end
        self.deactivateFocusFunction = function(focus, data)
             focus:Deactivate()
            self:UpdateBorderHighlight(focus, not ACTIVE)
        end
		
        local typeEntry 	= {control = self.typeList}
        local solventEntry = {control = self.solventList}
        local trait_1Entry	= {control = self.traitLists[1]}
        local trait_2Entry	= {control = self.traitLists[2]}
        local trait_3Entry	= {control = self.traitLists[3]}
		
        self.focusEntryData = {typeEntry, solventEntry, trait_1Entry, trait_2Entry, trait_3Entry}
        for _, v in pairs(self.focusEntryData) do
            v.activate = v.activate or self.activateFocusFunction
            v.deactivate = self.deactivateFocusFunction
        end
        self:RefreshFocusItems()
    end
    function IJA_GamepadEasyAlchemy:RefreshFocusItems(focusIndex)
         local entries = self.focusEntryData
		
        self.focus:RemoveAllEntries()
        for _, v in pairs(entries) do
            self.focus:AddEntry(v)
        end
        if focusIndex then
            self.focus:SetFocusByIndex(focusIndex)
        else
            self.focus:SetFocusByIndex(1)
        end
    end
	
    function IJA_GamepadEasyAlchemy:InitializeScrollPanel()
         local create = self.panelControl:GetNamedChild("Container")

        create:SetParent(self.scrollChild)
		
        create:ClearAnchors()
        create:SetAnchor(TOPLEFT, self.scrollChild, TOPLEFT)
        create:SetAnchor(TOPRIGHT, self.scrollChild, TOPRIGHT)
		
        self.panelControl:GetNamedChild("ScrollContainerScroll"):SetHandler("OnScrollExtentsChanged", function(...) self:OnScrollExtentsChanged(...) end)

		self:RefreshScrollPanel()
    end
    function IJA_GamepadEasyAlchemy:RefreshScrollPanel()
         local lists = {self.typeList, self.solventList, self.traitLists[1], self.traitLists[2], self.traitLists[3]}
		
        for _, entry in pairs(lists) do
            self:UpdateBorderHighlight(entry, not ACTIVE)
        end
    end
	
    function IJA_GamepadEasyAlchemy:OnRefreshAllLists()
         self:RefreshScrollPanel()
        local focusIndex = self.focus:GetFocus()
        self:RefreshFocusItems(focusIndex)
    end
end
function IJA_GamepadEasyAlchemy:OnScrollExtentsChanged(scroll, horizontalExtents, verticalExtents)
 	if not SCENE_MANAGER:IsShowing("gamepad_alchemy_creation") or SCENE_MANAGER:IsShowing("ija_gamepad_alchemy_saves") then return end
    -- rescroll visible area if necessary
    if verticalExtents > 0 then
        self:UpdateScrollPanel(self.currentFocus)
    end
end

do
    local SCROLL_PADDING_OFFSET = ZO_GAMEPAD_SMITHING_CONTAINER_ITEM_PADDING_Y * 2
    function IJA_GamepadEasyAlchemy:UpdateScrollPanel(focus)
         self.currentFocus = focus
        local focusControlParent = focus:GetControl():GetParent()
        local scrollTop = self.scrollContainer:GetTop() + SCROLL_PADDING_OFFSET
        local scrollBottom = self.scrollContainer:GetBottom() - SCROLL_PADDING_OFFSET
        local controlTop = focusControlParent:GetTop()
        local controlBottom = focusControlParent:GetBottom()
        local controlHeight = focusControlParent:GetHeight() + ZO_GAMEPAD_SMITHING_CONTAINER_ITEM_PADDING_Y
        if controlTop <= scrollTop then
            while(controlTop <= scrollTop) do
                ZO_ScrollRelative(self.scrollContainer, -controlHeight)
                controlTop = controlTop + controlHeight
            end
        elseif controlBottom >= scrollBottom then
            while(controlBottom >= scrollBottom) do
                ZO_ScrollRelative(self.scrollContainer, controlHeight)
                controlBottom = controlBottom - controlHeight
            end
        end
    end
end

function IJA_GamepadEasyAlchemy:UpdateBorderHighlight(focus, active)
     local focusControlParent = focus:GetControl():GetParent()
    focusControlParent.inactiveBG:SetHidden(active)
    focusControlParent.activeBG:SetHidden(not active)
	

	local universalItemBg = focusControlParent.universalItemBg
	universalItemBg:SetColor(ZO_CROWN_CRATES_GEMIFY_COLOR_FLASH:UnpackRGBA())
	universalItemBg:SetAlpha(0.1) -- STYLE_LIST_USI_BG_STANDARD_ALPHA = 0.35
		
	universalItemBg:SetHidden(not active)
end

function IJA_GamepadEasyAlchemy_Initialize( ... )
 	return IJA_GamepadEasyAlchemy:New( ... )
end

-------------------------------------
-- Saved Alchemy
-------------------------------------
local SAVED_LIST_SORT = {
	filterType = { tiebreaker = "meetsUsageRequirement", tieBreakerSortOrder = ZO_SORT_ORDER_DOWNN, isNumeric = true },
	meetsUsageRequirement = { tiebreaker = "name", tieBreakerSortOrder = ZO_SORT_ORDER_UP, isNumeric = true },
	name = {tiebreaker = "encodedAlchemyTraits", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
	encodedAlchemyTraits = { isNumeric = true },
}

function IJA_GamepadSavedAlchemyList:New(owner, control)
--[[
	local newObject = ZO_GamepadCraftingInventory.New(self, owner, control)
	newObject:Initialize(owner, control)
	return newObject
--]]
    return ZO_GamepadCraftingInventory.New(self, owner, control)
end

function IJA_GamepadSavedAlchemyList:Initialize(owner, control)
	ZO_GamepadCraftingInventory.Initialize(self, control)
	self.selectedData = {
		['type'] = 1,
		['solvent'] = {},
		['traitIds'] = {[1] = 1}
	}
	
	self:InitializeSolventList()
	self:InitializeKeybindStripDescriptors()
	self:InitializeScenes()
	
	self.filterType = NO_FILTER
	
	self.list:SetOnSelectedDataChangedCallback(function(list, selectedData, oldData, selectedDuringRebuild )
		if selectedData ~= nil then
			self.selectedData.type		= selectedData.filterType
			self.selectedData.traitIds	= selectedData.traitIds
			
			self:UpdateTooltip(self:GetTooltipData(selectedData.filterType == 2))
			KEYBIND_STRIP:UpdateKeybindButtonGroup(self.savedKeybindStripDescriptor)
		end
	end)
	
	self:GetList().CalculateParametricOffset = function(self, startAdditionalPadding, endAdditionalPadding, distanceFromCenter, continuousParametricOffset)
		local additionalPaddingEasingFunc
		if startAdditionalPadding < endAdditionalPadding then
			additionalPaddingEasingFunc = ZO_LinearEase
		end
		return ZO_ParametricScrollList.CalculateParametricOffset(self, startAdditionalPadding, endAdditionalPadding, distanceFromCenter, continuousParametricOffset, additionalPaddingEasingFunc)
	end
	
	self:SetNoItemLabelText(GetString(SI_IJAEA_NOSAVES))
end

function IJA_GamepadSavedAlchemyList:SelectSave()
	self.selectedSave = self.list:GetSelectedIndex()
	local solventData = self.solventList:GetSelectedData()

	self.selectedData.solvent = solventData
	
	self:SetMemory("SetSave", self.selectedData.traitIds)
	self:SetMemory("selectedTraits", self.selectedData.traitIds)
	self:SetMemory("oldType", self.selectedData.filterType)
	
	self:GetAlchemyRecipes(self:GetScanData())
	SCENE_MANAGER:HideCurrentScene()
	KEYBIND_STRIP:UpdateKeybindButtonGroup(self.mainKeybindStripDescriptor)
end

function IJA_GamepadSavedAlchemyList:InitializeKeybindStripDescriptors()
 	-- Main keybind strip
    self.mainKeybindStripDescriptor =
    {
        alignment = KEYBIND_STRIP_ALIGN_LEFT,
       
        { -- select saved recipe
			name = GetString(SI_GAMEPAD_SELECT_OPTION),
			keybind = "UI_SHORTCUT_PRIMARY",
			visible = function()
				return self.list:GetTargetData() ~= nil
			end,
			enabled = function()
				return self.list:GetTargetData() ~= nil-- and notSameRecipe()
			end,
			callback = function()
				local selectedData = self.list:GetTargetData()
				self.selectedData.traitIds = selectedData.traitIds
				self.selectedData.type = selectedData.filterType
				
				SCENE_MANAGER:Push("gamepad_alchemy_select_solvent")
			end,
			visible = function()
				return SCENE_MANAGER:IsShowing('ija_gamepad_alchemy_saves')
			end,
        },

        {-- Perform craft
            name = function()
                local cost = GetCostToCraftAlchemyItem(GAMEPAD_ALCHEMY.solventSlot:GetBagAndSlot())
                return ZO_CraftingUtils_GetCostToCraftString(cost)
            end,
            keybind = "UI_SHORTCUT_SECONDARY",
            gamepadOrder = 1000,
            callback = function()
				if SCENE_MANAGER:IsShowing('ija_gamepad_alchemy_saves') and self.selectedSave then
					self.list:SetSelectedIndex(self.selectedSave, ALLOW_EVEN_IF_DISABLED, SKIP_ANIMATION)
				end
                GAMEPAD_ALCHEMY:Create(1)
            end,
            enabled = function()
                return GAMEPAD_ALCHEMY:ShouldCraftButtonBeEnabled() and not self:IsUpdateInProgress()
            end,
            sound = SOUNDS.GAMEPAD_ALCHEMY_BEGIN,
        },
        
        {-- Multicraft
            name = GetString(SI_GAMEPAD_CRAFT_MULTIPLE),
            keybind = "UI_SHORTCUT_QUATERNARY",
            gamepadOrder = 1010,
            callback = function()
                local itemLink, _ = GetAlchemyResultingItemLink(GAMEPAD_ALCHEMY:GetAllCraftingBagAndSlots())
                ZO_GamepadCraftingUtils_ShowMultiCraftDialog(GAMEPAD_ALCHEMY, itemLink)
            end,
            enabled = function()
                return GAMEPAD_ALCHEMY:ShouldMultiCraftButtonBeEnabled() and GAMEPAD_ALCHEMY:ShouldCraftButtonBeEnabled() and not self:IsUpdateInProgress()
            end,
        },
		
        {-- Save or Delete Recipe
			name = GetString(SI_IJAEA_DELETE_OPTION),
			keybind = "UI_SHORTCUT_TERTIARY",
            gamepadOrder = 1020,
			enabled = function() return not ZO_CraftingUtils_IsPerformingCraftProcess() end,
			callback = function()
				local selectedData = self.list:GetTargetData()
				self:DeleteSavedData(selectedData)
			end,
			visible = function()
				return self:IsShowingAlchemyCraftingScene()
			end,
        },
    }
    ZO_GamepadCraftingUtils_AddGenericCraftingBackKeybindsToDescriptor(self.mainKeybindStripDescriptor)
    ZO_CraftingUtils_ConnectKeybindButtonGroupToCraftingProcess(self.mainKeybindStripDescriptor)
	
	self.solventKeybindStripDescriptor =
	{
		alignment = KEYBIND_STRIP_ALIGN_LEFT,
		-- select solvent
		{
			name = GetString(SI_GAMEPAD_SELECT_OPTION),
			keybind = "UI_SHORTCUT_PRIMARY",
			visible = function()
				return self.list:GetTargetData() ~= nil
			end,
			enabled = function()
				return self.list:GetTargetData() ~= nil
			end,
			callback = function()
				self:SelectSave()
			end,
		},
	}
	
	ZO_GamepadCraftingUtils_AddGenericCraftingBackKeybindsToDescriptor(self.solventKeybindStripDescriptor)
    ZO_CraftingUtils_ConnectKeybindButtonGroupToCraftingProcess(self.solventKeybindStripDescriptor)

end

function IJA_GamepadSavedAlchemyList:InitializeScenes()
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE = GAMEPAD_ALCHEMY:CreateInteractScene("ija_gamepad_alchemy_saves")
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE:AddFragment(IJA_RESULTPANEL_FRAGMENT)
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE:AddFragment(IJA_GAMEPAD_ALCHEMY_SAVELIST_FRAGMENT)
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE:AddFragmentGroup(IJA_GAMEPAD_EASYALCHEMY_FRAGMENTGROUP)
	
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE:AddFragment(GAMEPAD_ALCHEMY_SLOTS_FRAGMENT)
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE:SetInputPreferredMode(INPUT_PREFERRED_MODE_ALWAYS_GAMEPAD)
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE:AddFragmentGroup(FRAGMENT_GROUP.GAMEPAD_DRIVEN_UI_WINDOW)

	------------------------------------------------------------------------------------------------------------
	IJA_GAMEPAD_ALCHEMY_SAVED_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		if newState == SCENE_SHOWING then
			self:SetResultPanelHeader(GetString(SI_IJAEA_EASYALCHEMY))
	
			IJA_RESULTPANEL_FRAGMENT:Hide()
			self.list:Activate()
			ZO_GamepadAlchemyTopLevelHeaderContainerHeaderTitleContainerTitle:SetText(GetString(SI_IJAEA_SAVEDRECIPES))
		elseif newState == SCENE_SHOWN then
			self:SetStationFromMemory("ModeMemory_2")
			self:RefreshSolvents()
			
			KEYBIND_STRIP:RemoveDefaultExit()
			self:AddKeybindButtonGroup(self.mainKeybindStripDescriptor)
			
		elseif newState == SCENE_HIDDEN then
			self:SetStationMemory("ModeMemory_2")
			self.list:Deactivate()

			KEYBIND_STRIP:RemoveKeybindButtonGroup(self.mainKeybindStripDescriptor)
			KEYBIND_STRIP:RestoreDefaultExit()

			self:HandleDirtyEvent()
			GAMEPAD_CRAFTING_RESULTS:SetCraftingTooltip(nil)
			GAMEPAD_ALCHEMY.tooltip:SetHidden(true)
		end
	end)

	self:AddAlchemyCraftingScene("ija_gamepad_alchemy_saves")
------------------------------------------------------------------------------------------------------------
	IJA_GAMEPAD_ALCHEMY_SOLVENT_SCENE = GAMEPAD_ALCHEMY:CreateInteractScene("gamepad_alchemy_select_solvent")
	IJA_GAMEPAD_ALCHEMY_SOLVENT_SCENE:AddFragment(IJA_GAMEPAD_ALCHEMY_SOLVENT_FRAGMENT)
	IJA_GAMEPAD_ALCHEMY_SOLVENT_SCENE:AddFragmentGroup(IJA_GAMEPAD_EASYALCHEMY_FRAGMENTGROUP)
	IJA_GAMEPAD_ALCHEMY_SOLVENT_SCENE:SetInputPreferredMode(INPUT_PREFERRED_MODE_ALWAYS_GAMEPAD)
	IJA_GAMEPAD_ALCHEMY_SOLVENT_SCENE:AddFragmentGroup(FRAGMENT_GROUP.GAMEPAD_DRIVEN_UI_WINDOW)
------------------------------------------------------------------------------------------------------------
	IJA_GAMEPAD_ALCHEMY_SOLVENT_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		if newState == SCENE_SHOWING then
			KEYBIND_STRIP:RemoveDefaultExit()
			KEYBIND_STRIP:AddKeybindButtonGroup(self.solventKeybindStripDescriptor)
			
			self.solventList:Activate()
			
			IJA_GamepadSavedAlchemySolventTopLevel:SetHidden(false)
			self:RefreshSolventList()
		elseif newState == SCENE_HIDDEN then
			KEYBIND_STRIP:RemoveKeybindButtonGroup(self.solventKeybindStripDescriptor)
			KEYBIND_STRIP:RestoreDefaultExit()
			self.solventList:Deactivate()
			IJA_GamepadSavedAlchemySolventTopLevel:SetHidden(true)
		end
	end)
	
	self:AddAlchemyCraftingScene("gamepad_alchemy_select_solvent")
end

function IJA_GamepadSavedAlchemyList:HandleDirtyEvent()
    if self.control:IsHidden() or ZO_CraftingUtils_IsPerformingCraftProcess() then
        self.dirty = true
    else
        self:PerformFullRefresh()
    end
end

function IJA_GamepadSavedAlchemyList:InitializeSolventList()
     local scrollListControl = IJA_AlchemyHorizontalScrollList_Gamepad

    local listContainer = IJA_GamepadSavedAlchemySolventTopLevel:GetNamedChild("ContainerSolventList")
	
    listContainer:GetNamedChild("Label"):SetText(GetString("SI_IJAEA_EASYALCHEMY_HEADER", 2))

    local function SetupFunction(control, data, selected, selectedDuringRebuild, enabled)
 	
		----------------------------------------------
        ZO_ItemSlot_SetupSlot(control, 1, data.icon, true, not enabled)
        if selected then
            listContainer.selectedLabel:SetText(zo_strformat(SI_SMITHING_SELECTED_PATTERN, data.name))
			self:SetLabelHidden(listContainer.extraInfoLabel, true)
        end
    end

    local function EqualityFunction(leftData, rightData)
         return leftData.traitId == rightData.traitId
    end

    local function OnHorizonalScrollListCleared(...)
         self:OnHorizonalScrollListCleared(...)
    end

    local solventList = scrollListControl:New(listContainer.listControl, "ZO_GamepadSmithingListSlot", 5, SetupFunction, EqualityFunction, OnHorizonalScrollListShown, OnHorizonalScrollListCleared)
	solventList:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedSolventDataChangedCallback(selectedData, oldData, selectedDuringRebuild)
--[[
		if not selectedDuringRebuild then
			self.selectedData.solvent = selectedData
			self.selectedData.type = selectedData.solventType
			self:SetMemory("solventLevel", selectedData.craftingLevel)
		end

		local resultLink = self:GetResultLinks(selectedData.itemId, self.selectedData.traits)
		self:UpdatePanel({resultLink = resultLink})
		
		GAMEPAD_ALCHEMY:SetSolventItem(selectedData.bagId, selectedData.slotIndex)
		
		--]]
    end)
	
    solventList:SetScaleExtents(MIN_SCALE, MAX_SCALE)
	solventList.displayEntryType = ZO_HORIZONTAL_SCROLL_LIST_ANCHOR_ENTRIES_AT_FIXED_DISTANCE
	solventList.offsetBetweenEntries = 50
	
--	scrollList:SetMouseEnabled(true)

	self.solventList = solventList
	
    ZO_CraftingUtils_ConnectHorizontalScrollListToCraftingProcess(solventList)
	
    self.movementController = ZO_MovementController:New(MOVEMENT_CONTROLLER_DIRECTION_VERTICAL)
	self.focus = ZO_GamepadFocus:New(listContainer:GetParent())
end

function IJA_GamepadSavedAlchemyList:RefreshSolventList()
 	local solventDataList = self:RefreshSolvents()
	
	self.solventList:Clear()
	for k, solventData in pairs(solventDataList[self.selectedData.type]) do
		self.solventList:AddEntry(solventData)
	end
	self.solventList:Commit()

----------------------------------------------------------------------------------------------------
	self:SetSolventListIndex(self.solventList)
end

function IJA_GamepadSavedAlchemyList:AddListDataTypes()
	local function SetupTrait(controls, data)
		local numTraits = #data.traits
		for i, traitControl in ipairs(controls) do
			if i > numTraits then
				traitControl:SetHidden(true)
			else
				local iconControl = traitControl.icon or traitControl  -- For unlabeled trait controls, the control is the icon
				traitControl:SetHidden(false)

				local label = traitControl.label
				local traitName = data.traits[i]
				local traitIcon = self:GetTraitIcon(data.traitIds[i])
				if traitName then
					if label then
						label:SetText(traitName)
					end
				   GAMEPAD_ALCHEMY:SetupTraitIcon(iconControl, traitName, traitIcon)
				end
				if label then
					ZO_ItemSlot_SetupTextUsableAndLockedColor(label, data.meetsUsageRequirement, not data.meetsUsageRequirement)
				end
			end
		end
	end
	local function SetupSaveListEntry(control, data, selected, selectedDuringRebuild, enabled, activated)
		ZO_SharedGamepadEntry_OnSetup(control, data, selected, selectedDuringRebuild, enabled, activated)
		
		control.selectedItems:SetHidden(not data.meetsUsageRequirement or not selected)
		control:GetNamedChild("Label"):SetFont("ZoFontWinH3")
		control:GetNamedChild("Label"):SetWidth(300)
		
		
		if selected then
			SetupTrait(control.selectedItems.traits, data)
			control.selectedItems:SetHeight(30 * #data.traits)
		end

		ZO_ItemSlot_SetupTextUsableAndLockedColor(control, data.meetsUsageRequirement)
--		ZO_ItemSlot_SetupTextUsableAndLockedColor(control, data.meetsUsageRequirement, isLocked(data))
	end

	self.list:AddDataTemplate("IJA_GamepadSavedAlchemyListSlotTemplate", SetupSaveListEntry, ZO_GamepadMenuEntryTemplateParametricListFunction)
	self.list:AddDataTemplateWithHeader("IJA_GamepadSavedAlchemyListSlotTemplate", SetupSaveListEntry, ZO_GamepadMenuEntryTemplateParametricListFunction, nil, "ZO_GamepadMenuEntryHeaderTemplate")
end

function IJA_GamepadSavedAlchemyList:Refresh()
 	local savedAlchemyRecipes = ZO_DeepTableCopy(self.savedVars.savedAlchemyRecipes)
	
	table.sort(savedAlchemyRecipes, function(left, right)
		return ZO_TableOrderingFunction(left, right, "filterType", SAVED_LIST_SORT, ZO_SORT_ORDER_DOWN)
	end)
	
	self.list:Clear()

	local lastBestItemCategoryName
	for solventType = 1, 2 do
		for k,data in pairs(savedAlchemyRecipes[solventType]) do
			local name, icon, filterType, traitNames, traitIds, resultLink, meetsUsageRequirement, encodedAlchemyTraits, categoryName = self:GetDataForEntry(data)

			local entryData = ZO_GamepadEntryData:New(name, icon)
			
			entryData:SetAlphaChangeOnSelection(true)
			
			entryData.name					= name
			entryData.filterType			= filterType
			entryData.traits				= traitNames
			entryData.traitIds				= traitIds
			entryData.resultLink			= resultLink
			entryData.encodedAlchemyTraits	= encodedAlchemyTraits
			entryData.meetsUsageRequirement	= meetsUsageRequirement

				if categoryName ~= lastBestItemCategoryName then
					lastBestItemCategoryName = categoryName

					entryData:SetHeader(lastBestItemCategoryName)
					self.list:AddEntry("IJA_GamepadSavedAlchemyListSlotTemplateWithHeader", entryData)
				else
					self.list:AddEntry("IJA_GamepadSavedAlchemyListSlotTemplate", entryData)
				end
			if meetsUsageRequirement then
			end
		end
	end
	self.list:Commit()
end

function IJA_GamepadSavedAlchemyList:GetList()
 	return self.list
end

function IJA_GamepadSavedAlchemyList:GetTooltipData()
	local traitIds = self.selectedData.traitIds
	local solventData = self.solventList:GetSelectedData()
	local solventId = solventData and solventData.itemId or self:GetSolventByLevel(self.selectedData.type == 2)
	return self.selectedData.type, traitIds, solventId
end

function IJA_GamepadSavedAlchemyList:GetScanData()
	return self.selectedData.type, self.selectedData.traitIds, self.selectedData.solvent
end

--[[
/script SCENE_MANAGER:Push("ija_gamepad_alchemy_saves")
--]]