IJA_ALCHEMY_FILTER_TYPE = 1
IJA_ALCHEMY_FILTER_SOLVENT = 2
IJA_ALCHEMY_FILTER_TRAIT = 3
IJ_ALCHEMY_MODE_SAVES = 3

local MODE_GP = 1
local MODE_KB = 2


IJA_ACTIVEWRITS = IJA_ACTIVEWRITS or {}

local defaults = {
	displayName = "|cFF00FFIsJusta|r |cffffffEasy Alchemy|r",
	name = "IsJustaEasyAlchemy",
	author = "|cFF00FFIsJustaGhost|r",
	version = "2.2.6",
	character = false, 
}

local svVersion = 2.24

-------------------------------------
-- 
-------------------------------------
local IJA_EasyAlchemy = LibIJA:Subclass()

function IJA_EasyAlchemy:Subclass()
	local object = ZO_Object:Subclass()
	setmetatable( object, { __index = self } )
	return object
end

--[[

    local object = ZO_Object.New(self)
    object:Initialize(...)
    return object
--]]
function IJA_EasyAlchemy:New( ... )
	self:Initialize( ... )
	return self
end

function IJA_EasyAlchemy:Initialize(control)
	self.control = control
	self.control:RegisterForEvent( EVENT_ADD_ON_LOADED, function( ... ) self:OnLoaded( ... ) end )
end

function IJA_EasyAlchemy:OnLoaded(event, addon)
	self.control:UnregisterForEvent(EVENT_ADD_ON_LOADED)
	self.control:RegisterForEvent(EVENT_PLAYER_ACTIVATED, function(eventCode, ...) self:OnPlayerActivated(...) end)
	
	self.displayName	= defaults.displayName
	self.name 			= defaults.name
	self.version 		= defaults.version
	self.savedVarsVersion = defaults.savedVarsVersion
	self.mode = {}
	self.selectedData = {
		['type'] = 1,
		['solvent'] = {},
		['traitIds'] = {[1] = 1}
	}
	
	local AccountWideSavedVars = ZO_SavedVars:NewAccountWide("IJA_EasyAlchemy_SavedVars", svVersion, nil, defaults, GetWorldName())

	if AccountWideSavedVars.character then
		local characterSavedVars = ZO_SavedVars:New("IJA_EasyAlchemy_SavedVars", svVersion, nil, defaults, GetWorldName())
		self.savedVars = characterSavedVars
	else
		self.savedVars = AccountWideSavedVars
	end
	
	if not self.savedVars.savedAlchemyRecipes then self.savedVars.savedAlchemyRecipes = {[1] = {}, [2] = {}} end

-------------------------------------
	local function onCommandEntered(args)
	end

	SLASH_COMMANDS["/ijaea"] = onCommandEntered
	
-------------------------------------
	local deleteSavedDataDialogue = {
		gamepadInfo = { dialogType = GAMEPAD_DIALOGS.BASIC, allowShowOnNextScene = true },
		title = { text = GetString(SI_IJAEA_SAVEDALCHEMY_CONFIRMDELETE_TITLE) },
		mainText = { text =  function(dialog)
			if dialog.data then
				local nameColor = GetItemQualityColor(GetItemLinkDisplayQuality(dialog.data.itemLink))
				local itemName = nameColor:Colorize(dialog.data.recipeName)
				
				return zo_strformat(GetString(SI_IJAEA_SAVEDALCHEMY_CONFIRMDELETE_TEXT), itemName)
			else
				return GetString(SI_IJAEA_INVALIDRECIPE)
			end
		end},
		subText = { -- used in gamepade mode only :(
			align = TEXT_ALIGN_CENTER,
			text =  function(dialog)
				if dialog.data then
					local traits = dialog.data.traits
					local plural = #traits > 1 and 2 or 1
					return zo_strformat(GetString("SI_IJAEA_SAVEDALCHEMY_CONFIRMDELETE_TRAIT", plural), traits[1], traits[2], traits[3])
				end
			end,
		}, 
		buttons = {
			{
				text = SI_DIALOG_YES,
				callback = function(dialog)
					local savedAlchemyRecipes = self.savedVars.savedAlchemyRecipes[dialog.data.filterType]
					if dialog.data then	
						local mode = 0
						table.remove(savedAlchemyRecipes, dialog.data.index)
						if IsInGamepadPreferredMode() then
							self.mode[MODE_GP].saves:Refresh()
							mode = GAMEPAD_ALCHEMY.modeList:GetTargetData().mode
						else
							self.mode[MODE_KB].saves:HandleDirtyEvent()
							mode = ALCHEMY.mode
						end
				--		showHidePanels(mode)
					end
				end
			},
			{ text = SI_DIALOG_NO }
		}
	}
	ZO_Dialogs_RegisterCustomDialog("IJAEA_DELETE", deleteSavedDataDialogue)

	local function OnGamepadPreferredModeChanged()
		local gamepadMode = IsInGamepadPreferredMode()
		if gamepadMode then
			-- selective mode initialization. only initializes the mode if input mode is used
			if not self.mode[MODE_GP] then self.mode[MODE_GP] = IJA_GamepadEasyAlchemy_Initialize( self ) end
		elseif not self.mode[MODE_KB] then self.mode[MODE_KB] = IJA_KeyboardEasyAlchemy_Initialize( self ) end
	end
	ZO_PlatformStyle:New(OnGamepadPreferredModeChanged)
	
	local function QuestInformationUpdated()
		local mode = IsInGamepadPreferredMode() and MODE_GP or MODE_KB
		self.mode[mode]:RefreshAllListsLayer()
	end
	
	CRAFT_ADVISOR_MANAGER:RegisterCallback("QuestInformationUpdated", function(updatedQuestInfo) 
		self.alchemyQuestInfo = updatedQuestInfo.alchemyInfo
		QuestInformationUpdated()
	end)
end

function IJA_EasyAlchemy:OnPlayerActivated()
	self.control:UnregisterForEvent(EVENT_PLAYER_ACTIVATED)
	d( self.displayName .. " version: " .. self.version)
	
	IJA_EasyAlchemySettings_Initialize( self )
end

-------------------------------------
-- 
-------------------------------------
function IJA_EasyAlchemy:SetStationMemory(key)
	self:SetMemory(key, 
		{
			["traitIds"]		= self.selectedData.traitIds,
			["solventType"]		= self.selectedData.type,
			["solventLevel"]	= self:GetMemory("solventLevel")
		}
	)
end

function IJA_EasyAlchemy:SetStationFromMemory(key)
	local memory = self:GetMemory(key)

	if memory then
		local traitIds = memory.traitIds
		local solventType = memory.solventType
		local solventLevel = memory.solventLevel
		
		local oldType = self:GetMemory("oldType")
		local oldTraits = self:GetMemory("selectedTraits")
		if not self:AreSameTraits(oldTraits, traitIds) and solventType ~= oldType then
			return false
		end
		if solventType ~= oldType then
			return false
		end
		
		self.selectedData.traitIds	= traitIds
		self.selectedData.type		= solventType
		self:SetMemory("solventLevel", solventLevel)
		
		self:GetAlchemyRecipes(self:GetScanData())
		
		self:UpdateKeybindStrip()
	else
	end
end

function IJA_EasyAlchemy:OnHorizonalScrollListCleared(list)
	 local listContainer = list:GetControl():GetParent()
	listContainer.selectedLabel:SetHidden(true)
	self:SetLabelHidden(listContainer.extraInfoLabel, true)
end

function IJA_EasyAlchemy:RefreshSolventList(selectedData)
	local solventDataList = self:RefreshSolvents()
	local questItemIndex = 0
	self.solventList:Clear()
	for k, solventData in pairs(solventDataList[selectedData.type]) do
		if solventData then
			solventData.questPinState = self:GetQestPinState(nil, nil, solventData)
			if solventData.questPinState then questItemIndex = solventData.craftingLevel end
			self.solventList:AddEntry(solventData)
		end
	end
	self.solventList:Commit()

	self:SetSolventListIndex(self.solventList, questItemIndex)
end

function IJA_EasyAlchemy:OnSelectedSolventDataChangedCallback(selectedData, oldData, selectedDuringRebuild)
 	local solventType = selectedData.solventType
	self.selectedData.type = solventType
	self.selectedData.solvent = selectedData
	
	local supressSound = oldData == nil
	self:TryAddSolventToCraft(selectedData, supressSound, false)
	
	if oldData ~= nil then
		self:SetMemory("solventLevel", selectedData.craftingLevel)
		local itemLink = self:GetAlchemyResultLink(solventType, self.selectedData.traitIds, selectedData.itemId)
		self:UpdatePanel(itemLink, selectedData.itemId)
	end
	
	self:UpdateTooltip(self:GetTooltipData())
end

function IJA_EasyAlchemy:OnSelectedTraitDataChangedCallback(control, traitIndex, selectedData, oldData, selectedDuringRebuild)
 	local focusIndex = self.focus and self.focus:GetFocus() or nil
	local lastSelectedName = focusIndex and self:GetMemory(focusIndex) or nil
	
	local traitId = selectedData.traitId
	
	if not selectedDuringRebuild then
		self:SetMemory(traitIndex, traitId)
		
	end
	
	local function setTraitList(traitIndex, trait)
		for i = traitIndex+1 , 3 do
			self.selectedData.traitIds[i] = nil
		end
 		self.selectedData.traitIds[traitIndex] = trait ~= 0 and trait or nil
	end
	self:SetMemory("selectedTraits", ZO_ShallowTableCopy(self.selectedData.traitIds))
	
	if traitIndex == 1 then
		setTraitList(1, traitId)
		self:RefreshTraitLists(2, selectedData)
	elseif traitIndex == 2 then
		setTraitList(2, traitId)
		self:RefreshTraitLists(3, selectedData)
	elseif traitIndex == 3 then
 		self.selectedData.traitIds[traitIndex] = traitId ~= 0 and traitId or nil
	end
	
	self.selectedData.recipe = nil
	self.lockedItems = {}
	
	if not control:IsMoving() then
	end

	self:GetAlchemyRecipes(self:GetScanData())
	self:UpdateTooltip(self:GetTooltipData())
	self:UpdateKeybindStrip()

	if #self.lockedItems > 0 then
		for k,v in pairs(self.lockedItems) do
			d( v)
		end
	end
end

local function shouldAddTrait(labUse)
	if labUse  then
		return ZO_Alchemy_IsThirdAlchemySlotUnlocked()
	end
	return true
end

function IJA_EasyAlchemy:GetQuestInfo()
	return self.alchemyQuestInfo or nil
end

function IJA_EasyAlchemy:GetQestPinState(key, traitId, solventData)
	local alchemyQuestInfo = self:GetQuestInfo()
	if alchemyQuestInfo and not alchemyQuestInfo.hasDesiredPotion then
	
		local isPoison = self.selectedData.type == 2 and true or false
		if self:GetBaseIsPoison(alchemyQuestInfo.basePotionItemId) ~= isPoison then return false end
		
		local mode = IsInGamepadPreferredMode() and MODE_GP or MODE_KB
		local desiredTraits = {}
		if traitId then
			if alchemyQuestInfo then
			   if alchemyQuestInfo.isMasterWrit and ZO_Alchemy_IsThirdAlchemySlotUnlocked() then
					local traits = self:GetTraitsFromEncodedAlchemyTraits(alchemyQuestInfo.encodedTraits)
					for key,trait in pairs(traits) do
						desiredTraits[trait] = true
					end
				end
				if alchemyQuestInfo.desiredTrait then
					desiredTraits[alchemyQuestInfo.desiredTrait] = true
				end
			end
			
			if desiredTraits == nil then return false end
			
			if key == 1 then
				return desiredTraits[traitId]
			else
				local trait1 = self.mode[mode].traitLists[1]:GetSelectedData().traitId
				if key == 2 then
					if traitId ~= trait1 then
						return desiredTraits[traitId]
					end
				elseif key == 3 then
					local trait2 = self.mode[mode].traitLists[2]:GetSelectedData().traitId
					if traitId ~= trait1 and traitId ~= trait2 then
						return desiredTraits[traitId]
					end
				end
			end
		else
			if alchemyQuestInfo.basePotionItemId and alchemyQuestInfo.materialItemId then
				return IsAlchemySolventForItemAndMaterialId(solventData.bagId, solventData.slotIndex, alchemyQuestInfo.basePotionItemId, alchemyQuestInfo.materialItemId)
			end
		end
	end
	return false
end

function IJA_EasyAlchemy:RefreshTraitLists(key, selectedData)
	local function getTraitName(i)
 		if i == 0 then return end
		if self.selectedData.type == 1 then
			return self.POTIONTRAIT_DATA[i].name
		else
			return self.POTIONTRAIT_DATA[i].name2
		end
	end
	local questTraitId = 0
	local traitCombinations = self:GetTraitCombinations()

	self.traitLists[key]:Clear()
	if key == 1 then
		for i=1, #traitCombinations do
			local traitList = traitCombinations[i]
			local trait = traitList.trait
			local questPinState = self:GetQestPinState(key, trait)
			if questPinState == true then questTraitId = trait end
			self.traitLists[key]:AddEntry({traitId = trait, name = getTraitName(trait), icon = self:GetTraitIcon(trait), filterType = IJA_ALCHEMY_FILTER_TRAIT, questPinState = questPinState, Table = traitList})
		end
	else
		for i=1, #selectedData.Table do
			local traitList = selectedData.Table[i]
			local trait = traitList.trait
			if shouldAddTrait(traitList.labUse) then
				local questPinState = selectedData.questPinState and self:GetQestPinState(key, trait) or false
				if questPinState == true and questTraitId == 0 then questTraitId = trait end
				local name = trait > 0 and getTraitName(trait) or ""
				self.traitLists[key]:AddEntry({traitId = trait, name = getTraitName(trait), icon = self:GetTraitIcon(trait), filterType = IJA_ALCHEMY_FILTER_TRAIT, questPinState = questPinState, Table = traitList})
			end
		end	
	end
	self.traitLists[key]:Commit()
	
	local lastTraitId = questTraitId ~= 0 and questTraitId or self:GetMemory(key)
	local initialListIndex = 0
	if lastTraitId then
		local index = self.traitLists[key]:FindIndexFromData(lastTraitId, function(oldTraitId, newData)
			return oldTraitId == newData.traitId
		end)
		if index then
			initialListIndex = index
		end
	end
	local ALLOW_EVEN_IF_DISABLED = true
	local SKIP_ANIMATION = false
	
	self.traitLists[key]:SetSelectedIndex(initialListIndex, ALLOW_EVEN_IF_DISABLED, SKIP_ANIMATION)
end

function IJA_EasyAlchemy:GetMemoryTable()
     if not self.alchemySelectionMemory then
        self.alchemySelectionMemory = {}
    end
    return self.alchemySelectionMemory
end
function IJA_EasyAlchemy:GetMemory(key)
     local memory = self:GetMemoryTable()
    return memory[key]
end
function IJA_EasyAlchemy:SetMemory(key, data)
     local memory = self:GetMemoryTable()
    memory[key] = data
end

function IJA_EasyAlchemy:GetTooltipData()
	local traitIds = self.selectedData.traitIds or {[1] = 1}
	
	if self.activeTraitIds ~= nil then
		traitIds = activeTraitIds
	end
	
	return self.selectedData.type, traitIds, self.selectedData.solvent.itemId
end

function IJA_EasyAlchemy:GetScanData()
	return self.selectedData.type, self.selectedData.traitIds, self.selectedData.solvent
end

function IJA_EasyAlchemy:GetSavedVariables()
	return self.savedVars
end

function IJA_EasyAlchemy:GetSolventByLevel(isPoison)
	local skillRank = self:GetSkillRank(CRAFTING_TYPE_ALCHEMY, 1)

    local solventId
    if isPoison then
        solventId = select(skillRank, {75357, 75358}, 75359, 75360, 75361, 75362, 75363, 75364, 75365)
    else
        solventId = select(skillRank, {883, 1187}, 4570, 23265, 23266, 23267, 23268, 64500, 64501)
    end
	
	if type(solventId) == "table" then	-- for alchemy level 1
		local playerLevel = GetUnitLevel("player")
		solventId = playerLevel < 10 and solventId[1] or solventId[2]
	end
	return solventId
end

function IJA_EasyAlchemy:SetActiveTraitIds(traitIds)
	self.activeTraitIds = traitIds
end

--[[

--	self:GetAlchemyRecipes(self:GetScanData(), OnScanComplete)
--	self:GetAlchemyRecipes(resultType, traits, OnScanComplete)


	questInfo.conditionDat[2].itemFunctionalQuality

GetAlchemyWritLevel()

function lib:GetAlchemyWritFunctionalQuality()
	local questInfo = CRAFT_ADVISOR_MANAGER.questMasterList[CRAFT_ADVISOR_MANAGER.selectedMasterListIndex]
	for k, conditionInfo in pairs(questInfo.conditionData) do
		if conditionInfo.materialItemId then
			return conditionInfo.itemFunctionalQuality
		end
	end
end


|H1:item:64501:364:50:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:10000:0|h|h

|H1:item:27036:364:50:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:10000:0|h|h



Returns: string levelsDescription

/script d(GetItemLinkRequiredLevel("|H1:item:883:364:50:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:10000:0|h|h"))

/script d(GetItemLinkRequiredLevel("|H1:item:64501:364:50:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:10000:0|h|h"))

/script d(CRAFT_ADVISOR_MANAGER.questMasterList[CRAFT_ADVISOR_MANAGER.selectedMasterListIndex].level)
local itemFunctionalQuality = self:GetAlchemyWritFunctionalQuality()

		local desiredTraitId = GetTraitIdFromBasePotion(conditionInfo.itemId)


selectedData.questPinState
		self:ClearRecipeList()
		self:RefreshSolvents()
		self:ShowAlchemyResultPanel()

		self.tooltip:SetHidden(true)
		self.tooltip.tip:ClearLines()
		self.tooltip.tip:LayoutAlchemyPreview('', '', PROSPECTIVE_ALCHEMY_RESULT_UNKNOWN)
		
/script d(IJA_EASYALCHEMY.alchemyQuestInfo)
--]]
-------------------------------------
-- Saved Alchemy Locals
-------------------------------------
local SAVED_LIST_SORT = {
	filterType = { tiebreaker = "meetsUsageRequirement", tieBreakerSortOrder = ZO_SORT_ORDER_DOWNN, isNumeric = true },
	meetsUsageRequirement = { tiebreaker = "traits", tieBreakerSortOrder = ZO_SORT_ORDER_UP, isNumeric = true },
	traits = { tiebreaker = "name", tieBreakerSortOrder = ZO_SORT_ORDER_UP, isNumeric = true },
	name = {tiebreaker = "numTraits", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
	numTraits = { isNumeric = true },
}

local function normalizeName(link, filterType)
	-- done to append "Potion" to all potions, instead of having a level such as "Sip of"
	-- remove suffix from name
	local name = zo_strformat(SI_UNIT_NAME,GetItemLinkName(link)):gsub("^%l", string.upper)

	if filterType == 2 then
		-- remove poison level from name
		return name:gsub("%s%a+$", "")
	end
	
	return name
end

local function meetsUsageRequirement(itemLink)
	local icon, _, meetsUsageRequirement = GetItemLinkInfo(itemLink)
	return meetsUsageRequirement
end

-------------------------------------
-- Saved Alchemy Main
-------------------------------------
function IJA_EasyAlchemy:GetDataForEntry(data)
	local filterType	= data.filterType
	local traitIds		= data.traitIds
	local resultLink	= data.resultLink
	
	local name = normalizeName(resultLink, filterType)
	
	local solventList = self:RefreshSolvents()
	local skillRank = self:GetSkillRank(CRAFTING_TYPE_ALCHEMY, 1)
	
	for k,v in pairs(solventList[filterType]) do
		if skillRank == v.craftingLevel then
			self.selectedData.solvent = v
		end
	end

	local encodedAlchemyTraits = self:GetEncodedAlchemyTraits(traitIds)
	local meetsUsageRequirement = meetsUsageRequirement(self:UpdateItemLinkLevel(resultLink))
	
	local icon = GetItemLinkIcon(resultLink)
	
	local categoryName = filterType == 1 and GetString(SI_ITEMTYPEDISPLAYCATEGORY22) or GetString(SI_ITEMTYPEDISPLAYCATEGORY23)

	local entryData = ZO_GamepadEntryData:New(name, icon)
	local traitNames = self:GetEntryTraitNames(data.traitIds, filterType)
	
	return name, icon, filterType, traitNames, traitIds, resultLink, meetsUsageRequirement, encodedAlchemyTraits, categoryName
end

function IJA_EasyAlchemy:IsRecipeSaved(filterType, newTraits)
	local savedList = self.savedVars.savedAlchemyRecipes[filterType]
	if not savedList then return false end
	for i=1, #savedList do
		if self:AreSameTraits(savedList[i].traits, newTraits) then
			return true
		end
	end
	return false
end

function IJA_EasyAlchemy:DeleteSavedData(selectedData)
	if selectedData == nil then return end
	local savedAlchemyRecipes = self.savedVars.savedAlchemyRecipes[selectedData.filterType]
	for index, saveData in pairs(savedAlchemyRecipes) do
		if self:AreSameTraits(saveData.traitIds, selectedData.traitIds) then
			ZO_Dialogs_ShowPlatformDialog("IJAEA_DELETE", {recipeName = selectedData.name, index = index, itemLink = selectedData.resultLink, filterType = selectedData.filterType, traits = selectedData.traits })
			return
		end
	end
end

function IJA_EasyAlchemy:SaveCurrentRecipe()
	local traitIds = ZO_ShallowTableCopy(self.selectedData.traitIds)
	if self:IsRecipeSaved(self.selectedData.type, traitIds) then return end
	
	local solventId = self.selectedData.type == 1 and 23265 or 75360
	local resultLink = self:GetAlchemyResultLink(self.selectedData.type, traitIds, solventId)
	
	local data = {
		filterType = self.selectedData.type,
		traitIds = traitIds,
		resultLink = resultLink,
	}
	
	table.insert(self.savedVars.savedAlchemyRecipes[self.selectedData.type], data)
	data = nil
	ZO_Alert(UI_ALERT_CATEGORY_ALERT, SOUNDS.EDIT_CLICK, SI_IJAEA_SAVEDALCHEMY_SAVED)
	
	table.sort(self.savedVars.savedAlchemyRecipes, function(left, right)
		return ZO_TableOrderingFunction(left, right, "filterType", SAVED_LIST_SORT, ZO_SORT_ORDER_DOWN)
	end)

	self:UpdateKeybindButton()
end

function IJA_EasyAlchemy:IsSaveButtonEnabled()
	local newTraits = self.selectedData.traitIds
	local solventType = self.selectedData.type
	if self:IsUpdateInProgress() then
		return false
	end
	return not ZO_CraftingUtils_IsPerformingCraftProcess() and not self:IsRecipeSaved(solventType, newTraits)
end

function IJA_EasyAlchemy:IsSaveButtonVisible(mode)
	return mode == ZO_ALCHEMY_MODE_CREATION
end

function IJA_EasyAlchemy:SetLabelHidden(label, hidden)
	if hidden then
		label:SetText("")
	end
end

function IJA_EasyAlchemy:DirtyAllLists()
end

function IJA_EasyAlchemy:DirtyResult()
end

function IJA_EasyAlchemy:SetSolventListIndex(solventList, questItemIndex)
	local lastCraftingLevel = questItemIndex ~= 0 and questItemIndex or self:GetMemory("solventLevel")
	
	local initialListIndex = 0
	if lastCraftingLevel then
		local index = solventList:FindIndexFromData(lastCraftingLevel, function(oldCraftingLevel, newData)
			return oldCraftingLevel == newData.craftingLevel
		end)
		if index then
			initialListIndex = index
		end
	else
		local skillRank = self:GetSkillRank(CRAFTING_TYPE_ALCHEMY, 1)
		local index = solventList:FindIndexFromData(skillRank, function(skillRank, newData)
			return skillRank == newData.craftingLevel
		end)
		if index then
			initialListIndex = index
			self:SetMemory("solventLevel", skillRank)
		end
	end
	
	local ALLOW_EVEN_IF_DISABLED = true
	local SKIP_ANIMATION = false
	solventList:SetSelectedIndex(initialListIndex, ALLOW_EVEN_IF_DISABLED, SKIP_ANIMATION)
end

function IJA_EasyAlchemy:RefreshAllLists()
	local mode = IsInGamepadPreferredMode() and MODE_GP or MODE_KB
	self.mode[mode]:RefreshAllListsLayer()
end

-------------------------------------
-- 
-------------------------------------
function IJA_EasyAlchemy:ShouldCraftButtonBeEnabled(object)
	if ZO_CraftingUtils_IsPerformingCraftProcess() then
		return false
	end
	
	if not object.solventSlot:HasItem() then
		return false, GetString("SI_TRADESKILLRESULT", CRAFTING_RESULT_INVALID_BASE)
	end
	local numSlotsWithItems = 0
	for i, slot in ipairs(object.reagentSlots) do
		if slot:HasItem() and slot:MeetsUsabilityRequirement() then
			numSlotsWithItems = numSlotsWithItems + 1
		end
	end
	
	local REQUIRED_SLOTTED_REAGENTS = self.reagentList:GetSelectedData() and #self.reagentList:GetSelectedData().reagents or 2
	if numSlotsWithItems < REQUIRED_SLOTTED_REAGENTS then
		return false, GetString("SI_TRADESKILLRESULT", CRAFTING_RESULT_TOO_FEW_REAGENTS)
	end
	local _, prospectiveAlchemyResult = GetAlchemyResultingItemLink(object:GetAllCraftingBagAndSlots())
	if prospectiveAlchemyResult == PROSPECTIVE_ALCHEMY_RESULT_UNCRAFTABLE then
		-- allow invalid crafts, even if there isn't inventory space for it
		return true
	end
	
	if self:IsUpdateInProgress() then
		return false
	end
	
	local maxIterations, craftingResult = GetMaxIterationsPossibleForAlchemyItem(object:GetAllCraftingBagAndSlots())
	if SCENE_MANAGER:GetCurrentScene():GetName() == "alchemy" then
		return maxIterations >= object:GetMultiCraftNumIterations(), GetString("SI_TRADESKILLRESULT", craftingResult)
	else
		return maxIterations  ~= 0, GetString("SI_TRADESKILLRESULT", craftingResult)
	end
end

function IJA_EasyAlchemy:UpdateKeybindButton()---------------------------
	if IsInGamepadPreferredMode() then 
		self.mode[MODE_GP]:UpdateKeybindStrip()
	else
--		self.save_KB::UpdateKeybindStrip()
	end
end

function IJA_EasyAlchemy:AddKeybindButton()------------------------------
	if IsInGamepadPreferredMode() then
	else
		if not KEYBIND_STRIP:HasKeybindButton(IJA_KeyboardSavedAlchemy.saveKeybindButton) then
			KEYBIND_STRIP:AddKeybindButton(IJA_KeyboardSavedAlchemy.saveKeybindButton)
		end
	end
	self:UpdateKeybindButton()
end

function IJA_EasyAlchemy:RemoveKeybindButton()---------------------------
	if IsInGamepadPreferredMode() then 
		if KEYBIND_STRIP:HasKeybindButton(IJA_GamepadSavedAlchemy.saveKeybindButton) then
			KEYBIND_STRIP:RemoveKeybindButton(IJA_GamepadSavedAlchemy.saveKeybindButton)
			KEYBIND_STRIP:UpdateKeybindButtonGroup(GAMEPAD_ALCHEMY.mainKeybindStripDescriptor)
		end
	else
		if KEYBIND_STRIP:HasKeybindButton(IJA_KeyboardSavedAlchemy.saveKeybindButton) then
			KEYBIND_STRIP:RemoveKeybindButton(IJA_KeyboardSavedAlchemy.saveKeybindButton)
			KEYBIND_STRIP:UpdateKeybindButtonGroup(ALCHEMY.keybindStripDescriptor)
		end
	end
end

function IJA_EasyAlchemy_Initialize( ... )
	IJA_EASYALCHEMY =  IJA_EasyAlchemy:New( ... )
end


--[[
self.alchemyQuestInfo


/script d(IJA_EASYALCHEMY:GetQuestInfo())
/script d(type(IJA_EASYALCHEMY.alchemyQuestInfo))


	-- copy old saved recipes in case savedVars version change
	if IJA_EasyAlchemy_SavedVars then
		local savedAlchemyRecipes = {}
		for server,serverData in pairs(IJA_EasyAlchemy_SavedVars) do
			for account,accountData in pairs(serverData) do
				for _,v in pairs(accountData) do
					table.insert(savedAlchemyRecipes, v.savedAlchemyRecipes)
				end
			end
		end
		defaults.savedAlchemyRecipes = savedAlchemyRecipes
	end
	
	
	
--]]