local selectedSavedRecipe -- used for storing selected list entry control for unmarking it as selected if another is selected

local function areAlchemyPanelsEnabled()
	return not IJA_ACTIVEWRITS[CRAFTING_TYPE_ALCHEMY]
end

local IJA_KeyboardEasyAlchemy = IJA_EASYALCHEMY:Subclass()
local IJA_KeyboardSavedAlchemyList = IJA_KeyboardEasyAlchemy:Subclass()

function IJA_KeyboardEasyAlchemy:New(owner)
	local newObject = ZO_Object:MultiSubclass(self, owner)
	newObject:Initialize(owner)
	return newObject
end

function IJA_KeyboardEasyAlchemy:Initialize(owner)
	self.control = IJA_KeyboardEasyAlchemyPanel
	self.control.create = self.control:GetNamedChild("ContainerCreate")
	
	self.control.saves = self.control:GetNamedChild("ContainerSaves")
	
--	self.owner = owner
--	self.savedVars = owner.savedVars
--	self.savedVars = self:GetSavedVariables()
	
	self.traitLists = {}
	self.selectedData = {
		['type'] = 1,
		['solvent'] = {},
		['traitIds'] = {[1] = 1}
	}
	
	ALCHEMY_SCENE:AddFragment(ZO_FadeSceneFragment:New(IJA_KeyboardEasyAlchemyPanel))
	-- This refresh group is relatively new to the creation screen, and many
	-- things that could be considered refresh groups aren't reflected here yet.
	-- When possible, please consider organizing the existing refresh behaviors
	-- into a hierarchy that can then be added as a refresh group.
	self.refreshGroup = ZO_OrderedRefreshGroup:New(ZO_ORDERED_REFRESH_GROUP_AUTO_CLEAN_PER_FRAME)
	self.refreshGroup:AddDirtyState("AllLists", function()
		self:RefreshAllListsLayer()
	end)
	self.refreshGroup:AddDirtyState("Result", function()
		self:RefreshResultLayer()
	end)
	self.refreshGroup:SetActive(function()
--		return SCENE_MANAGER:IsShowing("ija_gamepad_alchemy_creation")
	end)

	local alchemyScene = SCENE_MANAGER:GetScene("alchemy")
	alchemyScene:RegisterCallback("StateChange", function(oldState, newState)
		if newState == SCENE_SHOWING then
			self.tooltip = self.keyboardTooltip
			self:PerformDeferredInitialization()
		elseif newState == SCENE_SHOWN then
			KEYBIND_STRIP:RemoveKeybindButtonGroup(ALCHEMY.keybindStripDescriptor)
			alchemyScene:UnregisterCallback("StateChange")
		elseif newState == SCENE_HIDDEN then
		end
	end)
end

function IJA_KeyboardEasyAlchemy:InitializeModeBar()
	local creationData = {
		activeTabText = GetString(SI_IJAEA_SAVEDRECIPES),
		categoryName = GetString(SI_IJAEA_SAVEDRECIPES),
		descriptor = IJ_ALCHEMY_MODE_SAVES,
		normal = "esoui/art/inventory/inventory_tabicon_consumables_up.dds",
		pressed = "esoui/art/inventory/inventory_tabicon_consumables_down.dds",
		highlight = "esoui/art/inventory/inventory_tabicon_consumables_over.dds",
		disabled = "esoui/art/inventory/inventory_tabicon_consumables_disabled.dds",
--		visible = function() return isTabVisable() end,
--		enabled = function() return IJAEA.isEasyAlchemy end,
		callback = function(creationData)
			ALCHEMY.modeBarLabel:SetText(GetString(SI_IJAEA_SAVEDRECIPES))
			ALCHEMY:SetMode(creationData.descriptor)
		end
	}
	ZO_MenuBar_AddButton(ALCHEMY.modeBar, creationData)

	local newOrder = {}
	newOrder[1] = ALCHEMY.modeBar.m_object.m_buttons[1]
	newOrder[2] = ALCHEMY.modeBar.m_object.m_buttons[3]
	newOrder[3] = ALCHEMY.modeBar.m_object.m_buttons[2]

	ALCHEMY.modeBar.m_object.m_buttons = newOrder
	ZO_MenuBar_UpdateButtons(ALCHEMY.modeBar)
end

function IJA_KeyboardEasyAlchemy:InitializeHooks()
	local function isNotActiveWrit()
		if IJA_ACTIVEWRITS then 
			return not IJA_ACTIVEWRITS[CRAFTING_TYPE_ALCHEMY]
		end
		-- refresh
		return true
	end
	local isLocal = false
	
	local function showElements(mode)
--		IJA_AlchemyTopLevel:SetHidden(mode == ZO_ALCHEMY_MODE_RECIPES)
		
		IJA_KeyboardEasyAlchemyPanel:SetHidden(mode == ZO_ALCHEMY_MODE_RECIPES)
		IJA_KeyboardEasyAlchemyPanelContainerCreate:SetHidden(mode ~= ZO_ALCHEMY_MODE_CREATION)
		IJA_KeyboardEasyAlchemyPanelContainerSaves:SetHidden(mode ~= IJ_ALCHEMY_MODE_SAVES)
		ALCHEMY.control:GetNamedChild("SlotContainer"):SetHidden(mode == ZO_ALCHEMY_MODE_RECIPES)
	end

	local function activateLists(activate, mode)
		for i=1, 3 do self.traitLists[i]:SetEnabled(activate and mode == ZO_ALCHEMY_MODE_CREATION) end
		self.solventList:SetEnabled(activate and  mode ~= ZO_ALCHEMY_MODE_RECIPES)
	end
	
	local SelectMode = {
		['mode'] = 2,
		['index'] = 2,
		['type'] = "SelectMode",
		['doFunc'] = function(mode, object)
--			if isNotActiveWrit() then
			if object.mode ~= mode then
				showElements(mode)
		--		activateLists(isNotActiveWrit(), mode)
				
				LIB_IJA_Alchemy:SetHidden(mode == ZO_ALCHEMY_MODE_RECIPES)
				if mode ~= ZO_ALCHEMY_MODE_RECIPES then
					object.control:GetNamedChild("Inventory"):SetHidden(true) -- hide the original alchemy crafting inventory list
					
					self.tooltip = self.keyboardTooltip
					if selectedSavedRecipe ~= nil then
						selectedSavedRecipe = nil
					end
					
					local oldMode = object.mode
					if oldMode and oldMode ~= ZO_ALCHEMY_MODE_RECIPES then
						self:SetStationMemory("ModeMemory_" .. oldMode)
					end
					if oldMode == ZO_ALCHEMY_MODE_RECIPES then
						PROVISIONER:RemoveFromCraftingScene()
						KEYBIND_STRIP:AddKeybindButtonGroup(object.keybindStripDescriptor)
					end
					
					self:SetStationFromMemory("ModeMemory_" .. mode)
					
					 LIB_IJA_Alchemy:SetHidden(false)

					if mode == ZO_ALCHEMY_MODE_CREATION then
						if isNotActiveWrit() then
							self:SetActiveTraitIds(self.selectedData.traitIds)
							self:ChangeTypeFilter({descriptor = self.typeFilter})
							self:RefreshTraitLists(1)
							
							self:SetResultPanelHeader(GetString(SI_IJAEA_EASYALCHEMY))
							self:AddKeybindButtonGroup(self.mainKeybindStripDescriptor)
						else
						end
					else -- mode is IJ_ALCHEMY_MODE_SAVES
						if self.saves.selectedSavedRecipe ~= nil then
							self.saves:ReselectSave()
						end
						
						self:SetActiveTraitIds(self.saves.selectedData.traitIds)
				
						ZO_MenuBar_SelectDescriptor(self.saves.tabs, self.saves.filterType)
						self:AddKeybindButtonGroup(self.mainKeybindStripDescriptor)
						self:SetResultPanelHeader(GetString(SI_IJAEA_SAVEDRECIPES))
						self.saves:SetActiveFilterByDescriptor(self.saves.typeFilter)
					end
					
					object.mode = mode
					isLocal = true
					return true
				end
			end
			isLocal = false
			return false
		end
	}
	self:AddCallBack(SelectMode)
	
	local StateChange = {
		['mode'] = 2,
		['index'] = 2,
		['type'] = "StateChange",
		['doFunc'] = function(oldState, newState)
			if newState == SCENE_SHOWING then
				KEYBIND_STRIP:RemoveKeybindButtonGroup(ALCHEMY.keybindStripDescriptor)
				ALCHEMY_SCENE:AddFragment(IJA_ALCHEMY_FRAGMENT)
			elseif newState == SCENE_SHOWN then
				ALCHEMY.tooltip:SetHidden(true)
				CRAFTING_RESULTS:SetCraftingTooltip(self.tooltip)
				CRAFTING_RESULTS:SetTooltipAnimationSounds(SOUNDS.ALCHEMY_CREATE_TOOLTIP_GLOW_SUCCESS, SOUNDS.ALCHEMY_CREATE_TOOLTIP_GLOW_FAIL)
				
				self:AddKeybindButtonGroup(self.mainKeybindStripDescriptor)
			elseif newState == SCENE_HIDING then
			elseif newState == SCENE_HIDDEN then
				ZO_InventorySlot_RemoveMouseOverKeybinds()
				KEYBIND_STRIP:RemoveKeybindButtonGroup(self.currentKeybindStripDescriptor)
				self.tooltip:SetHidden(true)
			end
			return true
		end
	}
	self:AddCallBack(StateChange)
end

function IJA_KeyboardEasyAlchemy:InitializeScenes()
	IJA_KEYBOARDEASYALCHEMYPANEL_FRAGMENT = ZO_CreateQuadrantConveyorFragment(self.control)
	ALCHEMY_SCENE:AddFragment(IJA_KEYBOARDEASYALCHEMYPANEL_FRAGMENT)
	ALCHEMY_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		if newState == SCENE_SHOWING then
			self:PerformDeferredInitialization()
			
			-- Reselect so we re-add the temporary fragment for the recipe mode
			-- and setup/update the tooltip and corresponding sounds correctly
			CRAFTING_RESULTS:SetCraftingTooltip(IJA_EASYALCHEMY.KB.tooltip)
			CRAFTING_RESULTS:SetTooltipAnimationSounds(SOUNDS.ALCHEMY_CREATE_TOOLTIP_GLOW_SUCCESS, SOUNDS.ALCHEMY_CREATE_TOOLTIP_GLOW_FAIL)
			
			local oldMode = self.mode
			self.mode = nil
			ZO_MenuBar_ClearSelection(self.modeBar)
			if not oldMode then
				ZO_MenuBar_SelectDescriptor(self.modeBar, ZO_ALCHEMY_MODE_CREATION)
			else
				ZO_MenuBar_SelectDescriptor(self.modeBar, oldMode)
			end
			if CRAFT_ADVISOR_MANAGER:HasActiveWrits() then
				SCENE_MANAGER:AddFragmentGroup(WRIT_ADVISOR_KEYBOARD_FRAGMENT_GROUP)
			end

		elseif newState == SCENE_HIDDEN then
			ZO_InventorySlot_RemoveMouseOverKeybinds()
			KEYBIND_STRIP:RemoveKeybindButtonGroup(self.currentKeybindStripDescriptor)
			
			IJA_KeyboardEasyAlchemyPanel:SetHidden(true)
			IJA_KeyboardEasyAlchemyPanelContainerCreate:SetHidden(true)
			IJA_KeyboardEasyAlchemyPanelContainerSaves:SetHidden(true)
			
			CRAFTING_RESULTS:SetCraftingTooltip(nil)
			SCENE_MANAGER:RemoveFragmentGroup(WRIT_ADVISOR_KEYBOARD_FRAGMENT_GROUP)
		end
	end)
end

function IJA_KeyboardEasyAlchemy:DisableClickOnSlot()
	 ALCHEMY.solventSlot:GetControl():SetMouseEnabled(false)
	 for i=1, #ALCHEMY.reagentSlots do
		ALCHEMY.reagentSlots[i]:GetControl():SetMouseEnabled(false)
	 end
end

function IJA_KeyboardEasyAlchemy:PerformDeferredInitialization()
	if self.mainKeybindStripDescriptor then return end
  	
	self:SetMemory("solventLevel", self:GetSkillRank(CRAFTING_TYPE_ALCHEMY, 1))
		
	local scrollListControl = ZO_HorizontalScrollList
	local listSlotTemplate = "ZO_SmithingListSlot"
	self:DisableClickOnSlot()
	self:InitializeSolventList()
	self:InitializeLists(scrollListControl, listSlotTemplate)
	
	self.saves = IJA_KeyboardSavedAlchemyList:New(self.control:GetNamedChild("ContainerSaves"), self)
	
	self:InitializeFilterTypeBar()
	ZO_MenuBar_SelectDescriptor(self.tabs, 1)
	
	self:InitializeMainKeybindStripDescriptors()
	
	ZO_InventoryInfoBar_ConnectStandardBar(self.control:GetNamedChild("InfoBar"))
	
	self:InitializeModeBar()
	self:InitializeHooks()
	
	self:InitializeMainKeybindStripDescriptors()
	
	if not self.tooltip then self.tooltip = ALCHEMY.tooltip end
	if IsChatSystemAvailableForCurrentPlatform() then
		local function OnTooltipMouseUp(control, button, upInside)
			if upInside and button == 2 then
				local link = ZO_LinkHandler_CreateChatLink(GetAlchemyResultingItemLink, self:GetAllCraftingBagAndSlots())
				if link ~= "" then
					ClearMenu()
					local function AddLink()
						ZO_LinkHandler_InsertLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, link))
					end
					AddMenuItem(GetString(SI_ITEM_ACTION_LINK_TO_CHAT), AddLink)
					ShowMenu(self)
				end
			end
		end
		self.keyboardTooltip:SetHandler("OnMouseUp", OnTooltipMouseUp)
		self.keyboardTooltip:GetNamedChild("Icon"):SetHandler("OnMouseUp", OnTooltipMouseUp)
	end
	
	ALCHEMY.mode = ZO_ALCHEMY_MODE_RECIPES
	zo_callLater(function()
		ALCHEMY:SetMode(ZO_ALCHEMY_MODE_CREATION)
	end, 50)
end

function IJA_KeyboardEasyAlchemy:InitializeMainKeybindStripDescriptors()
	-- Main keybind strip
	self.mainKeybindStripDescriptor =
	{
		alignment = KEYBIND_STRIP_ALIGN_CENTER,

			-- scan
		{
			name = GetString(SI_GAMEPAD_SELECT_OPTION),
			keybind = "UI_SHORTCUT_PRIMARY",
			visible = function() return true end,
			enabled = function()
				local oldTraits = self:GetMemory("selectedTraits")
				return not ZO_CraftingUtils_IsPerformingCraftProcess() and
				not self:AreSameTraits(oldTraits, self.selectedData.traitIds)
			end,
			callback = function()
				self:SetMemory("selectedTraits", ZO_ShallowTableCopy(self.selectedData.traitIds))
				self:SetMemory("oldType", self.selectedData.type)

				self:UpdateSettingsData(self.savedVars)
				self:ScanForRecipes(self.selectedData.traitIds)
				
				self:UpdateKeybindStrip()
			end,
			visible = function() return false end
		},

		-- Perform craft
		{
			name = function()
				local cost = GetCostToCraftAlchemyItem(ALCHEMY.solventSlot:GetBagAndSlot())
				return ZO_CraftingUtils_GetCostToCraftString(cost)
			end,
			keybind = "UI_SHORTCUT_SECONDARY",
			callback = function()
			   ZO_KeyboardCraftingUtils_RequestCraftingCreate(ALCHEMY, ALCHEMY:GetMultiCraftNumIterations())
			end,
			enabled = function()
				return ALCHEMY:ShouldCraftButtonBeEnabled() and not self:IsUpdateInProgress()
			end,
			sound = SOUNDS.GAMEPAD_ALCHEMY_BEGIN,
		},
		-- save selections
		{
			name = GetString(SI_SAVE),
			keybind = "UI_SHORTCUT_TERTIARY",
			visible = function()
				return true
			end,
			enabled = function()
				return self:IsSaveButtonEnabled()
			end,
			callback = function()
				self:SaveCurrentRecipe()
--				self:UpdateTooltip()
				
				self.saves:PerformFullRefresh()
				
				self:UpdateKeybindStrip()
			end,
			visible = function() return self:IsSaveButtonVisible(ALCHEMY.mode) end
		},
	}
	ZO_CraftingUtils_ConnectKeybindButtonGroupToCraftingProcess(self.mainKeybindStripDescriptor)
	
	-- /script d(IJA_EASYALCHEMY.KB.saves.list)
	self.savedKeybindStripDescriptor =
	{
		alignment = KEYBIND_STRIP_ALIGN_CENTER,
		-- Perform craft
		{
			name = function()
				local cost = GetCostToCraftAlchemyItem(GAMEPAD_ALCHEMY.solventSlot:GetBagAndSlot())
				return ZO_CraftingUtils_GetCostToCraftString(cost)
			end,
			keybind = "UI_SHORTCUT_SECONDARY",
			callback = function()
				ZO_KeyboardCraftingUtils_RequestCraftingCreate(self, ALCHEMY:GetMultiCraftNumIterations())
			end,
			enabled = function()
				return ALCHEMY:ShouldCraftButtonBeEnabled() and not self:IsUpdateInProgress()
			end,
			sound = SOUNDS.GAMEPAD_ALCHEMY_BEGIN,
		}
	}
	ZO_CraftingUtils_ConnectKeybindButtonGroupToCraftingProcess(self.savedKeybindStripDescriptor)
end

function IJA_KeyboardEasyAlchemy:SetHidden(hidden)
	self.control:SetHidden(hidden)
	if not hidden then
 --	   CRAFTING_RESULTS:SetCraftingTooltip(self.tooltip)
 --	   CRAFTING_RESULTS:SetTooltipAnimationSounds(self:GetCreateTooltipSound())
	end
	self.refreshGroup:TryClean()
end

function IJA_KeyboardEasyAlchemy:SetupResultTooltip(...)
--	self.tooltip:SetPendingSmithingItem(...)
end

function IJA_KeyboardEasyAlchemy:GetPlatformFormattedTextString(stringId, ...)
	return zo_strformat(stringId, ...)
end
--[[
function IJA_KeyboardEasyAlchemy:UpdateKeybindStrip()
	KEYBIND_STRIP:UpdateKeybindButtonGroup(self.currentKeybindStripDescriptor)
end
--]]
function IJA_KeyboardEasyAlchemy:ConfirmCreate()
	ZO_KeyboardCraftingUtils_RequestCraftingCreate(self, self:GetMultiCraftNumIterations())
end

function IJA_KeyboardEasyAlchemy:RefreshResultLayer()
 --   SMITHING_GAMEPAD.creationPanel:RefreshTooltip()
 --   SMITHING_GAMEPAD.creationPanel:RefreshMultiCraft()
 --   SMITHING_GAMEPAD.creationPanel:UpdateKeybindStrip()
end

function IJA_KeyboardEasyAlchemy:ChangeTypeFilter(filterData)
	self.typeFilter = filterData.descriptor
	
	self.selectedData.type = filterData.descriptor
	
	self:RefreshSolventList(self.selectedData)
	self:RefreshTraitLists(1)
	
	self:DirtyAllLists()
end

function IJA_KeyboardEasyAlchemy:DirtyAllLists()
	self.refreshGroup:MarkDirty("AllLists")
end

function IJA_KeyboardEasyAlchemy:DirtyResult()
	self.refreshGroup:MarkDirty("Result")
end

function IJA_KeyboardEasyAlchemy:OnRefreshAllLists()
	-- Need to do this on PC, as the first selection can be garbage otherwise with the style<->pattern cyclic dependencies.
	-- On gamepad, the list auto-scrolls to an item and refreshes twice on its own, so this isn't necessary.
	self:RefreshTraitLists(1)
end

local function OnHorizonalScrollListShown(list)
	local listContainer = list:GetControl():GetParent()
	listContainer.selectedLabel:SetHidden(false)
end
function IJA_KeyboardEasyAlchemy:OnHorizonalScrollListCleared(list)
	local listContainer = list:GetControl():GetParent()
	listContainer.selectedLabel:SetHidden(true)
	self:SetLabelHidden(listContainer.extraInfoLabel, true)
end

function IJA_KeyboardEasyAlchemy:RefreshAllListsLayer()
	if self.typeFilter then
		self.performingFullRefresh = true
		self:RefreshSolventList(self.selectedData)
--		for i=1, 3 do self.traitLists[i]:RefreshVisible() end
		self:RefreshTraitLists(1)
		self:RefreshAvailableFilters()
		self.performingFullRefresh = false
	end
--	self:OnRefreshAllLists()
end

function IJA_KeyboardEasyAlchemy:InitializeList(listName, TYPE, scrollListClass, listSlotTemplate)
	local listContainer = self.control.create:GetNamedChild(listName)
	
	listContainer.titleLabel:SetText(GetString("SI_IJAEA_EASYALCHEMY_HEADER", TYPE))

	local function SetupFunction(control, data, selected, selectedDuringRebuild, enabled)
	
		----------------------------------------------
		ZO_ItemSlot_SetupSlot(control, 1, data.icon, true, not enabled)
		if selected then
			listContainer.selectedLabel:SetText(zo_strformat(SI_SMITHING_SELECTED_PATTERN, data.name))
			self:SetLabelHidden(listContainer.extraInfoLabel, true)
			
		else
		end
		
		if TYPE > 2 then
			control:GetNamedChild('QuestPin'):SetHidden(not data.questPinState)
		end
	end

	local function EqualityFunction(leftData, rightData)
		return leftData.traitId == rightData.traitId
	end

	local function OnHorizonalScrollListCleared(...)
		self:OnHorizonalScrollListCleared(...)
	end

	scrollList = scrollListClass:New(listContainer.listControl, listSlotTemplate, 5, SetupFunction, EqualityFunction, OnHorizonalScrollListShown, OnHorizonalScrollListCleared)

	scrollList:SetScaleExtents(MIN_SCALE, MAX_SCALE)
	scrollList.displayEntryType = ZO_HORIZONTAL_SCROLL_LIST_ANCHOR_ENTRIES_AT_FIXED_DISTANCE
	scrollList.offsetBetweenEntries = 50
	
--	scrollList:SetMouseEnabled(true)

	ZO_CraftingUtils_ConnectHorizontalScrollListToCraftingProcess(scrollList)
	return scrollList
end

function IJA_KeyboardEasyAlchemy:InitializeLists(scrollListClass, listSlotTemplate)
	local trait_1List = self:InitializeList("Trait_1List", IJA_ALCHEMY_FILTER_TRAIT, scrollListClass, listSlotTemplate)
	trait_1List:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedTraitDataChangedCallback(trait_1List, 1, selectedData, oldData, selectedDuringRebuild)
	end)
	self.traitLists[1] = trait_1List
	self.traitLists[1].onPlaySoundFunction(ZO_HORIZONTALSCROLLLIST_MOVEMENT_TYPES.INITIAL_UPDATE)
	
	local trait_2List = self:InitializeList("Trait_2List", IJA_ALCHEMY_FILTER_TRAIT + 1, scrollListClass, listSlotTemplate)
	trait_2List:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedTraitDataChangedCallback(trait_2List, 2, selectedData, oldData, selectedDuringRebuild)
	end)
	self.traitLists[2] = trait_2List
	self.traitLists[2].onPlaySoundFunction(ZO_HORIZONTALSCROLLLIST_MOVEMENT_TYPES.INITIAL_UPDATE)
	
	local trait_3List = self:InitializeList("Trait_3List", IJA_ALCHEMY_FILTER_TRAIT + 2, scrollListClass, listSlotTemplate)
	trait_3List:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedTraitDataChangedCallback(trait_3List, 3, selectedData, oldData, selectedDuringRebuild)
--		self:UpdateTooltip(self.selectedData, true)
--	self:UpdateTooltip(self:GetTooltipData())
	end)
	self.traitLists[3] = trait_3List
	self.traitLists[3].onPlaySoundFunction(ZO_HORIZONTALSCROLLLIST_MOVEMENT_TYPES.INITIAL_UPDATE)
	
end

-------------- Solvent list
function IJA_KeyboardEasyAlchemy:InitializeSolventList()
	local scrollListControl = ZO_HorizontalScrollList

	local listContainer = IJA_KeyboardEasyAlchemyPanel:GetNamedChild("SolventList")
	
	listContainer:GetNamedChild("Label"):SetText(GetString("SI_IJAEA_EASYALCHEMY_HEADER", 2))

	local function SetupFunction(control, data, selected, selectedDuringRebuild, enabled)
	
		----------------------------------------------
		ZO_ItemSlot_SetupSlot(control, 1, data.icon, true, not enabled)
		if selected then
			listContainer.selectedLabel:SetText(zo_strformat(SI_SMITHING_SELECTED_PATTERN, data.name))
			listContainer.extraInfoLabel:SetText(data.stackCount)
			listContainer.extraInfoLabel:SetHidden(false)
		end
		control:GetNamedChild('QuestPin'):SetHidden(not data.questPinState)
	end

	local function EqualityFunction(leftData, rightData)
		return leftData.traitId == rightData.traitId
	end

	local function OnHorizonalScrollListCleared(...)
		self:OnHorizonalScrollListCleared(...)
	end

	local solventList = scrollListControl:New(listContainer.listControl, "ZO_GamepadSmithingListSlot", 5, SetupFunction, EqualityFunction, OnHorizonalScrollListShown, OnHorizonalScrollListCleared)
	solventList:SetOnSelectedDataChangedCallback(function(selectedData, oldData, selectedDuringRebuild)
		self:OnSelectedSolventDataChangedCallback(selectedData, oldData, selectedDuringRebuild)
	end)
	
	
	solventList:SetScaleExtents(MIN_SCALE, MAX_SCALE)
	solventList.displayEntryType = ZO_HORIZONTAL_SCROLL_LIST_ANCHOR_ENTRIES_AT_FIXED_DISTANCE
	solventList.offsetBetweenEntries = 50
	
--	scrollList:SetMouseEnabled(true)

	self.movementController = ZO_MovementController:New(MOVEMENT_CONTROLLER_DIRECTION_VERTICAL)
	self.focus = ZO_GamepadFocus:New(listContainer:GetParent())

	self.solventList = solventList
	
	ZO_CraftingUtils_ConnectHorizontalScrollListToCraftingProcess(solventList)
end

-------------- Trait lists
function IJA_KeyboardEasyAlchemy:RefreshAvailableFilters()
	ZO_MenuBar_ClearSelection(self.tabs)
	ZO_MenuBar_UpdateButtons(self.tabs)
	ZO_MenuBar_SelectLastVisibleButton(self.tabs, true)
	
	local alchemyQuestInfo = self:GetQuestInfo()
	
	local function isQuestActive(isPoison)
		if alchemyQuestInfo and not alchemyQuestInfo.hasDesiredPotion then
			
			local mode = self:GetBaseIsPoison(alchemyQuestInfo.basePotionItemId) and 2 or 1
			zo_callLater(function()
			ZO_MenuBar_SelectDescriptor(self.tabs, mode)
			end, 100)
			
			return self:GetBaseIsPoison(alchemyQuestInfo.basePotionItemId) ~= isPoison
		
		end
		return true
	end

	self.createPotionButton.questPin:SetHidden( isQuestActive(false))
	self.createPoisonButton.questPin:SetHidden( isQuestActive(true))
	
    if ALCHEMY.creationButton then
		local isHidden = true
		if alchemyQuestInfo ~= nil then
			isHidden = alchemyQuestInfo.hasDesiredPotion or false
		end
        ALCHEMY.creationButton.questPin:SetHidden(isHidden)
    end
end

function IJA_KeyboardEasyAlchemy:InitializeFilterTypeBar()
	local MENU_BAR_DATA =
	{
		initialButtonAnchorPoint = RIGHT,
		buttonTemplate = "ZO_CraftingInventoryTab",
		normalSize = 51,
		downSize = 64,
		buttonPadding = -15,
		animationDuration = 180,
	}

	self.tabs = self.control.create:GetNamedChild("Tabs")
	self.activeTab = self.tabs:GetNamedChild("Active")
	ZO_MenuBar_SetData(self.tabs, MENU_BAR_DATA)

	local function CreateNewTabFilterData(filterType, name, normal, pressed, highlight, disabled, visible)
		return {
			tooltipText = name,
			activeTabText = name,
			descriptor = filterType,
			normal = normal,
			pressed = pressed,
			highlight = highlight,
			disabled = disabled,
			visible = visible,
			callback = function(filterData)
				self.activeTab:SetText(name)
				self:ChangeTypeFilter(filterData)
			end,
		}
	end

	local function isVisible() return true end
	self.createPoisonButton = ZO_MenuBar_AddButton(self.tabs, CreateNewTabFilterData(2, GetString(SI_ITEMTYPEDISPLAYCATEGORY23), "/esoui/art/progression/progression_tabicon_combatskills_up.dds", "/esoui/art/progression/progression_tabicon_combatskills_down.dds", "/esoui/art/progression/progression_tabicon_combatskills_over.dds", "/esoui/art/progression/progression_tabicon_combatskills_disabled.dds", isVisible ))
	self.createPotionButton = ZO_MenuBar_AddButton(self.tabs, CreateNewTabFilterData(1, GetString(SI_ITEMTYPEDISPLAYCATEGORY22), "esoui/art/inventory/inventory_tabicon_consumables_up.dds", "esoui/art/inventory/inventory_tabicon_consumables_down.dds", "esoui/art/inventory/inventory_tabicon_consumables_over.dds", "esoui/art/inventory/inventory_tabicon_consumables_disabled.dds", isVisible ))
	
	ZO_MenuBar_UpdateButtons(self.tabs)
	ZO_CraftingUtils_ConnectMenuBarToCraftingProcess(self.tabs)
	self:RefreshAvailableFilters()
end

function IJA_KeyboardEasyAlchemyTabs_OnInitialized(control)
	ZO_MenuBar_OnInitialized(control)
	local filterBarData =
	{
		initialButtonAnchorPoint = RIGHT,
		buttonTemplate = "ZO_SmithingCreationFilterTab",
		normalSize = 40,
		downSize = 51,
--		buttonPadding = -5,
		buttonPadding = -10,
		animationDuration = 180,
	}
	ZO_MenuBar_SetData(control, filterBarData)
end

function IJA_KeyboardEasyAlchemy_Initialize( ... )
	return IJA_KeyboardEasyAlchemy:New( ... )
end

-------------------------------------
-- 
-------------------------------------
local SAVED_LIST_SORT = {
	filterType = { tiebreaker = "meetsUsageRequirement", tieBreakerSortOrder = ZO_SORT_ORDER_UP, isNumeric = true },
	meetsUsageRequirement = { tiebreaker = "name", tieBreakerSortOrder = ZO_SORT_ORDER_UP, isNumeric = true },
	name = {tiebreaker = "encodedAlchemyTraits", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
	encodedAlchemyTraits = { isNumeric = true },
}

local POTION_ENTRY	= 1
local POISON_ENTRY	= 2
local LIST_HEADER	= 3

function IJA_KeyboardSavedAlchemyList:New(...)
	return IJA_KeyboardSavedAlchemyList:Initialize(...)
end

local MENU_BAR_DATA =
{
	initialButtonAnchorPoint = RIGHT,
	buttonTemplate = "ZO_CraftingInventoryTab",
	normalSize = 51,
	downSize = 64,
	buttonPadding = -15,
	animationDuration = 180,
}

function IJA_KeyboardSavedAlchemyList:SetFilters(filterData)
	if not self.tabs then
		self.tabs = self.control:GetNamedChild("Tabs")
		self.activeTab = self.control:GetNamedChild("TabsActive")
		ZO_MenuBar_SetData(self.tabs, MENU_BAR_DATA)
		ZO_CraftingUtils_ConnectMenuBarToCraftingProcess(self.tabs)
	else
		ZO_MenuBar_ClearButtons(self.tabs)
	end
	for _, data in ipairs(filterData) do
		ZO_MenuBar_AddButton(self.tabs, data)
	end
end

function IJA_KeyboardSavedAlchemyList:Initialize(control, parent)
	self.control = control
--	self.owner = parent.owner
	self.parent = parent
--	self.savedVars = parent.savedVars
--	self.savedVars = self:GetSavedVariables()

	self.selectedData = {
		['type'] = 1,
		['solvent'] = {},
		['traitIds'] = {[1] = 1}
	}
	
	self:InitializeList()

	local function CreateNewTabFilterData(filterType, name, normal, pressed, highlight, disabled, visible)
		return {
			activeTabText = name,
			tooltipText = name,
			descriptor = filterType,
			normal = normal,
			pressed = pressed,
			highlight = highlight,
			disabled = disabled,
			visible = visible,
			callback = function(tabData) self:ChangeFilter(tabData) end,
		}
	end

	self:SetFilters{
		CreateNewTabFilterData(POISON_ENTRY, GetString(SI_ITEMTYPEDISPLAYCATEGORY23), "/esoui/art/tradinghouse/tradinghouse_potions_poisonsolvent_up.dds", "/esoui/art/tradinghouse/tradinghouse_potions_poisonsolvent_down.dds", "/esoui/art/tradinghouse/tradinghouse_potions_poisonsolvent_over.dds", "/esoui/art/tradinghouse/tradinghouse_potions_poisonsolvent_disabled.dds"),
		CreateNewTabFilterData(POTION_ENTRY, GetString(SI_ITEMTYPEDISPLAYCATEGORY22), "EsoUI/Art/Crafting/alchemy_tabIcon_solvent_up.dds", "EsoUI/Art/Crafting/alchemy_tabIcon_solvent_down.dds", "EsoUI/Art/Crafting/alchemy_tabIcon_solvent_over.dds", "EsoUI/Art/Crafting/alchemy_tabIcon_solvent_disabled.dds"),
		CreateNewTabFilterData(nil, GetString(SI_ITEMFILTERTYPE0), "EsoUI/Art/Inventory/inventory_tabIcon_all_up.dds", "EsoUI/Art/Inventory/inventory_tabIcon_all_down.dds", "EsoUI/Art/Inventory/inventory_tabIcon_all_over.dds", "EsoUI/Art/Inventory/inventory_tabIcon_all_disabled.dds"),
	}

	self:SetNoItemLabelHidden(false)
	
	local sortByHeaderControl = control:GetNamedChild("SortBy")
	local sortHeaderGroup = ZO_SortHeaderGroup:New(sortByHeaderControl, true)
	local function OnSortHeaderClicked(key, order) self:ChangeSort(key, order) end
	
	sortHeaderGroup:RegisterCallback(ZO_SortHeaderGroup.HEADER_CLICKED, OnSortHeaderClicked)
	sortHeaderGroup:AddHeadersFromContainer()
	self.sortHeaderGroup = sortHeaderGroup
	self.noDragging = true
	
	self.sortOrder = ZO_SORT_ORDER_UP
	self.filterSort = ZO_SORT_ORDER_UP

	self.sortHeaderGroup:SelectHeaderByKey("filterType", ZO_SortHeaderGroup.SUPPRESS_CALLBACKS, ZO_SortHeaderGroup.FORCE_RESELECT, ZO_SORT_ORDER_UP)
	
	self:HandleDirtyEvent()
	return self
end

function IJA_KeyboardSavedAlchemyList:HandleDirtyEvent()
	if self.control:IsHidden() or ZO_CraftingUtils_IsPerformingCraftProcess() then
		self.dirty = true
	else
		self:PerformFullRefresh()
	end
end

function IJA_KeyboardSavedAlchemyList:InitializeList()
	self.list = self.control:GetNamedChild("List")
	
--	ZO_ScrollList_EnableHighlight(self.list, "ZO_ListEntryHighlight")
	
	self.list:SetClampedToScreen(true)
	self.noItemsLabel = self.control:GetNamedChild("NoItemsLabel")
	self:AddListDataTypes()
end

function IJA_KeyboardSavedAlchemyList:SetActiveFilterByDescriptor(descriptor)
	if descriptor then
		self:ChangeFilter(descriptor)
	else
--		ZO_MenuBar_SelectDescriptor(self.tabs, nil)
--		self:ChangeFilter({descriptor = self.typeFilter})
	end
		ZO_MenuBar_SelectDescriptor(self.tabs, descriptor)
end

local function meetsUsageRequirement(itemLink)
	local icon, _, meetsUsageRequirement = GetItemLinkInfo(itemLink)
	return meetsUsageRequirement
end

local function isLocked(data)	--------------------
	return not meetsUsageRequirement(data.resultLink)
end

function IJA_ItemSlot_SetupSlotBase(slotControl, iconFile, visible)
	local showSlot = true
	if type(visible) == "function" then
		showSlot = visible()
	elseif visible ~= nil then
		showSlot = visible
	end
	slotControl:SetHidden(not showSlot)

	local iconControl = GetControl(slotControl, "Icon")
	if iconControl then
		if iconFile == nil or iconFile == "" then
			iconControl:SetHidden(true)
		else
			iconControl:SetTexture(iconFile)
			iconControl:SetHidden(false)
		end
	end
end
function IJA_KeyboardSavedAlchemyList_SetupUsableAndLockedColor(slotControl, meetsUsageRequirement, locked)
	ZO_ItemSlot_SetupTextUsableAndLockedColor(GetControl(slotControl, "Name"), meetsUsageRequirement, locked)
	ZO_ItemSlot_SetupUsableAndLockedColor(GetControl(slotControl, "Button"), meetsUsageRequirement, locked)
end
function IJA_KeyboardSavedAlchemyList_SetupSlot(slotControl, data)
	IJA_ItemSlot_SetupSlotBase(GetControl(slotControl, "Button"), data.icon, true)
	IJA_KeyboardSavedAlchemyList_SetupUsableAndLockedColor(slotControl, meetsUsageRequirement(data.resultLink), isLocked(data))
end

function IJA_KeyboardSavedAlchemyList:SetMousedOverRow(slot)
	self.mousedOverSlot = slot
	self.selectedData.solvent = self.selectedData.solvent
		
	if slot ~= nil then
		local data = slot.data
		local solventId = self.parent.solventList:GetSelectedData().itemId
		
		local traitIds = data.traitIds
		self:UpdateTooltip(data.filterType, traitIds, solventId)
		
		local func = function()
			ClearTooltip(InformationTooltip)
		end
		
		EVENT_MANAGER:UnregisterForUpdate("IJA_CraftingInventoryComponentRow")
		EVENT_MANAGER:RegisterForUpdate("IJA_CraftingInventoryComponentRow", 1000, func)
	else
		EVENT_MANAGER:UnregisterForUpdate("IJA_CraftingInventoryComponentRow")
		
		self:UpdateTooltip(self:GetTooltipData())
	end
end

function IJA_KeyboardSavedAlchemyList:ReselectSave()
	if self.selectedSavedRecipe ~= nil then
		local highlight = self.selectedSavedRecipe:GetNamedChild("SelectedHighlight")
		
		highlight:SetHidden(false)
		
		self:RefreshSolventList(self.selectedData)
		self.selectedData.solvent = self.selectedData.solvent
		
		self:GetAlchemyRecipes(self:GetScanData())
		self:UpdateTooltip(self:GetTooltipData())
	end
end

function IJA_KeyboardSavedAlchemyList:AddListDataTypes()
	local function toggleRow(control)
		local highlight = control:GetNamedChild("SelectedHighlight")
		
		if self.selectedSavedRecipe ~= nil then
			self.selectedSavedRecipe:GetNamedChild("SelectedHighlight"):SetHidden(true)
		end
		
		if self.selectedSavedRecipe == control then
			self.selectedSavedRecipe = nil
		else
			highlight:SetHidden(false)
			self.selectedSavedRecipe = control
		end
	end
	
	local function onSelected(control, button, data)
		if button == 1 then -- left-click
			if control ~= self.selectedSavedRecipe then
				self:SetMemory("SetSave", data.traitIds)
				self:SetMemory("selectedTraits", data.traitIds)
				self:SetMemory("oldType", data.filterType)
				
				self.selectedData.type = data.filterType
				self.selectedData.traitIds = data.traitIds
				
				self:RefreshSolventList(self.selectedData)
				self:GetAlchemyRecipes(self:GetScanData())
				
				toggleRow(control)
				self:UpdateKeybindStrip()
				
				self:SetActiveTraitIds(data.traitIds)
				
			end
		elseif button == 2 then	-- right-click
			self:DeleteSavedData(data)
		end
	end

	local function SetupSaveListEntry(control, data)
		local function SetupTraits(controls, data)
			local numTraits = #data.traits
			for i, traitControl in ipairs(controls) do
				if i > numTraits then
					traitControl:SetHidden(true)
				else
					traitControl:SetHidden(false)
					local traitName = data.traits[i]
					local traitIcon = IJA_EASYALCHEMY:GetTraitIcon(data.traitIds[i])
					
					if traitName and traitName ~= "" then
						traitControl.label:SetColor(GetInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_TOOLTIP, ITEM_TOOLTIP_COLOR_ACCENT))
						traitControl.label:SetText(traitName)

						ALCHEMY:SetupTraitIcon(traitControl.icon, traitName, traitIcon)
						traitControl.icon:SetHidden(false)
					else
						traitControl.label:SetColor(GetInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_TOOLTIP, ITEM_TOOLTIP_COLOR_INACTIVE_BONUS))
						traitControl.label:SetText(GetString(SI_CRAFTING_UNKNOWN_NAME))
						traitControl.icon:SetHidden(true)
					end
					ZO_ItemSlot_SetupTextUsableAndLockedColor(traitControl.label, data.meetsUsageRequirement, isLocked(data))
				end
			end
		end
		local function SetupRow(control, data)
			control.owner = self
			control.data = data
			
			local row = control:GetNamedChild("Button")
			row.name = row.name or control:GetNamedChild("Name")
			row.name:SetText(data.name)

			row.name:SetWidth(control:GetRight())

			IJA_KeyboardSavedAlchemyList_SetupSlot(control, data)
			row.tooltipAnchor = control

			row.owner = self
			ZO_UpdateTraitInformationControlIcon(control, data)

			self.noDragging = true
			if self.noDragging then
				control:SetHandler("OnDragStart", nil)
				control:SetHandler("OnReceiveDrag", nil)
			end

			control:SetHandler("OnMouseUp", function(_, button)
				onSelected(control, button, data)
			end)

			ZO_ItemSlot_SetupTextUsableAndLockedColor(row, data.meetsUsageRequirement, isLocked(data))
			
		end
			SetupRow(control, data)
			SetupTraits(control.traits, data)
	end
	
	local function setupSectionHeaderRow(control, data)
		control.header:SetText(data.text)
		control:SetHidden(false)
	end
	
	local LIST_HEIGHT = 450
	ZO_ScrollList_SetHeight(self.list, LIST_HEIGHT)
-- header
	ZO_ScrollList_AddDataType(self.list, LIST_HEADER, "IJA_KeyboardSavedAlchemyListHeaderTemplate", 32, setupSectionHeaderRow)
	ZO_ScrollList_AddDataType(self.list, POTION_ENTRY, "IJA_KeyboardSavedAlchemyListSlotTemplate", 108, SetupSaveListEntry)
	ZO_ScrollList_AddDataType(self.list, POISON_ENTRY, "IJA_KeyboardSavedAlchemyListSlotTemplate", 108, SetupSaveListEntry)
end

function IJA_KeyboardSavedAlchemyList:SetNoItemLabelText(text)
	self.noItemsLabel:SetText(text)
end
function IJA_KeyboardSavedAlchemyList:SetNoItemLabelHidden(isHidden)
	self.noItemsLabel:SetHidden(isHidden)
end

function IJA_KeyboardSavedAlchemyList:HandleDirtyEvent()
	if self.control:IsHidden() or ZO_CraftingUtils_IsPerformingCraftProcess() then
		self.dirty = true
	else
		self:PerformFullRefresh()
	end
end

function IJA_KeyboardSavedAlchemyList:ChangeFilter(filterData)
	ZO_CraftingInventory.ChangeFilter(self, filterData)
	self.filterType = filterData.descriptor
	
	self.sortHeaderGroup:SetHeadersHiddenFromKeyList({ filterType = self.filterType ~= nil }, true)
	
	if self.filterType then
		self:SetNoItemLabelText(GetString("SI_IJAEA_NOSAVES", self.filterType))
	else --	self.filterType == nil
		self:SetNoItemLabelText(GetString(SI_IJAEA_NOSAVES))
	end
	
	local solventType = self.filterType == 2 and 2 or 1 -- self.filterType can = nil
	self.selectedData.type = solventType
--	self:RefreshSolventList(self.selectedData)
	
	self:SetNoItemLabelHidden(false)
	self:HandleDirtyEvent()
end

function IJA_KeyboardSavedAlchemyList_DoesAlchemyItemPassFilter(solventType, filterType)
	if filterType == nil then
		return true
	end
	return filterType == solventType
end

function IJA_KeyboardSavedAlchemyList:RefreshFilters()
	ZO_MenuBar_UpdateButtons(self.tabs)
	if not ZO_MenuBar_GetSelectedDescriptor(self.tabs) then
		ZO_MenuBar_SelectLastVisibleButton(self.tabs, true)
	end
end

function IJA_KeyboardSavedAlchemyList:SortData()
	local scrollData = ZO_ScrollList_GetDataList(self.list)
	self.sortFunction = self.sortFunction or function(entry1, entry2)

		if entry1.data.filterType == entry2.data.filterType then
			if entry1.typeId ~= entry2.typeId then
				return entry1.typeId > entry2.typeId	-- force header to top of list
			end
			
			return ZO_TableOrderingFunction(entry1.data, entry2.data, self.sortKey, SAVED_LIST_SORT, self.sortOrder)
		end
		
		if self.filterSort then 
			return entry1.data.filterType < entry2.data.filterType
		else
			return entry1.data.filterType > entry2.data.filterType
		end
	end
	table.sort(scrollData, self.sortFunction)
end

function IJA_KeyboardSavedAlchemyList:ChangeSort(sortKey, sortOrder)
	if sortKey == "filterType" then 
		self.filterSort = not self.filterSort
	else
		self.sortOrder = not self.sortOrder
		self.sortKey = sortKey
	end
	
	self:SortData()
	ZO_ScrollList_Commit(self.list)
end

function IJA_KeyboardSavedAlchemyList:AddData(savedData)
	local name, icon, filterType, traitNames, traitIds, resultLink, meetsUsageRequirement, encodedAlchemyTraits = self.parent:GetDataForEntry(savedData)

	local newData = {
		name = name,
		icon = icon,
		filterType = filterType,
		traits = traitNames,
		traitIds = traitIds,
		resultLink = resultLink,
		encodedAlchemyTraits = encodedAlchemyTraits,
		meetsUsageRequirement = meetsUsageRequirement,
	}
	
	return newData
end

function IJA_KeyboardSavedAlchemyList:EnumerateAndAddToScrollData(filterFunction, filterType, scrollData)
	local lastCategory = 0
	for solventType = 1, 2 do
		if not filterFunction or filterFunction(solventType, filterType) then
			local savedAlchemyRecipes = ZO_DeepTableCopy(self.savedVars.savedAlchemyRecipes[solventType])
			if #savedAlchemyRecipes ~= 0 then
				if filterType == nil then		
					if lastCategory ~= solventType then
						local sectionTitleText = solventType == 1 and GetString(SI_ITEMTYPEDISPLAYCATEGORY22) or GetString(SI_ITEMTYPEDISPLAYCATEGORY23)
						scrollData[#scrollData + 1] = ZO_ScrollList_CreateDataEntry(LIST_HEADER, { text = sectionTitleText, filterType = solventType})
						lastCategory = solventType
					end
				end
				
				for index, savedData in pairs(savedAlchemyRecipes) do
					if savedData == nil then break end
					local newScrollData = self:AddData(savedData)
		
					if newScrollData.meetsUsageRequirement then
					end
					scrollData[#scrollData + 1] = ZO_ScrollList_CreateDataEntry(solventType, newScrollData)
				end
			end
		end
	end
end

function IJA_KeyboardSavedAlchemyList:PerformFullRefresh()
	self.dirty = false
	if not self.performingFullRefresh then
		self.performingFullRefresh = true
		ZO_ScrollList_Clear(self.list)
		self:RefreshFilters()
		self:Refresh(ZO_ScrollList_GetDataList(self.list))
		self:SortData()
		ZO_ScrollList_Commit(self.list)
		self.performingFullRefresh = false
		
		if selectedSavedRecipe ~= nil then
			selectedSavedRecipe:SetHidden(true)
			selectedSavedRecipe = nil
		end
	end
end

function IJA_KeyboardSavedAlchemyList:Refresh(scrollData)
	self:EnumerateAndAddToScrollData(IJA_KeyboardSavedAlchemyList_DoesAlchemyItemPassFilter, self.filterType, scrollData)
	self:SetNoItemLabelHidden(#scrollData > 0)
end

function IJA_KeyboardSavedAlchemyList:GetTooltipData()
	local traitIds = self.selectedData.traitIds
	local solventData = self.parent.solventList:GetSelectedData()
	local solventId = solventData and solventData.itemId or self:GetSolventByLevel(self.selectedData.type == 2)
	return self.selectedData.type, traitIds, solventId
end

function IJA_KeyboardSavedAlchemyList:GetScanData()
	return self.selectedData.type, self.selectedData.traitIds, self.selectedData.solvent
end

function IJA_KeyboardSavedAlchemyList:RefreshSolventList(...)
	self.parent:RefreshSolventList(...)
end


--[[
/script d(IJA_EASYALCHEMY.mode[2].saves.list.data)

/script d( GetItemLinkRequiredCraftingSkillRank("|H0:item:27041:308:50:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h"))


Search on ESOUI Source Code GetItemLinkRequiredCraftingSkillRank(string itemLink)
Returns: number requiredRank

|H0:item:30142:30:20:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:2162688|h|h


-- /script d("|H0:item:30142:30:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h")


-- /script d("|H0:item:27041:30:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h")
-- /script d("|H0:item:27041:308:50:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h")
-- /script d("|H0:item:27041:30:20:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:5915|h|h")


-- /script d("|H0:item:27041:30:20:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:2293760|h|h")
-- /script d("|H0:item:76848:30:20:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:2293760|h|h")

--]]